/*****************************************************************************
 *  Rohde&Schwarz RTH Instrument Driver
 *  LabWindows/CVI Instrument Driver
 *  Original Release: 2017-05-11
 *  By: Petr Kominek
 *
 *  Modification History:
 *
 *       See ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rsrth.h"
#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#if defined (_CVI_)
#include <utility.h>
#endif

/*****************************************************************************
 * Function: rsrth_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsrth_InitWithOptions function.
 *****************************************************************************/
ViStatus rsrth_init(ViRsrc resourceName, ViBoolean IDQuery,
    ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
            VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsrth_InitWithOptions(resourceName, IDQuery, resetDevice,
        "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrth_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus rsrth_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
            "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr*)rsrth_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsrth", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsrth_RsCoreAttributeListLength, // Attributes count
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSRTH_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.2.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 2, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsrth_RsCoreRepCapTable, rsrth_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "SMA100B", false : "SMA"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSRTH_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSRTH_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsrth_reset(instrSession));
    }
    else
    {
        checkErr(rsrth_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSRTH_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties != NULL && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)RsCore_CheckStatus(instrSession, error);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN  This function performs an autoset process: analyzes the enabled
/// HIFN  channel signals, and obtains appropriate horizontal, vertical, and
/// HIFN  trigger settings to display stable waveforms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_Autoset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_AUTOSCALE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the calibration operating mode. To determine the
/// HIFN  skew offset, use the 'Skew' operating mode. It uses the internal
/// HIFN  channel alignment for best results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the calibration operating mode. To determine the skew offset, use
/// HIPAR mode/the 'Skew' operating mode. It uses the internal channel alignment for
/// HIPAR mode/best results.
ViStatus rsrth_ConfigureMode(ViSession instrSession,
                             ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures  how the waveform is built from the captured
/// HIFN  samples.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionMode/Specify how the waveform is built from the captured samples.
ViStatus rsrth_ConfigureAcquisitionMode(ViSession instrSession,
                                        ViInt32   acquisitionMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SAMPLE_MODE, acquisitionMode),
            2, "Acquisition Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function the number of waveforms used to calculate the average
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofAverages/Pass the number  of waveforms used to calculate the average waveform.
ViStatus rsrth_ConfigureNumberOfAverages(ViSession instrSession,
                                         ViInt32   numberofAverages)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_NUM_AVERAGES, numberofAverages),
            2, "Number of Averages");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the record length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preselectedRecordLength/Sets the record length.
ViStatus rsrth_ConfigurePreselectedRecordLength(ViSession instrSession,
                                                ViInt32   preselectedRecordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PRESELECTED_RECORD_LENGTH, preselectedRecordLength),
            2, "Preselected Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The setting is relevant if the time scale is >= 50 ms/div. At these
/// HIFN  slow timebases, you can select how the acquired samples are displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformUpdate/The setting is relevant if the time scale is >= 50 ms/div. At these
/// HIPAR waveformUpdate/slow timebases, you can select how the acquired samples are displayed.
ViStatus rsrth_ConfigureWaveformUpdate(ViSession instrSession,
                                       ViInt32   waveformUpdate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WAVEFORM_UPDATE, waveformUpdate),
            2, "Waveform Update");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common attributes of the acquisition
/// HIFN  subsystem.  These attributes are the time per record and acquisition
/// HIFN  start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timePerRecord/Pass the time of one acquisition, that is the time across the 10
/// HIPAR timePerRecord/divisions of the diagram: Horizontal Scale *10.
/// HIPAR acquisitionStartTime/Sets the horizontal position of the trigger point in relation to the
/// HIPAR acquisitionStartTime/reference point.
ViStatus rsrth_ConfigureAcquisitionRecord(ViSession instrSession,
                                          ViReal64  timePerRecord,
                                          ViReal64  acquisitionStartTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_HORZ_TIME_PER_RECORD, timePerRecord),
            2, "Time Per Record");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_ACQUSITION_START_TIME, acquisitionStartTime),
            3, "Acquisition Start Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the acquisition time, the timerange across
/// HIFN  the 10 horizontal divisions of the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionTime/Defines the time of one acquisition, that is the time across the 10
/// HIPAR acquisitionTime/divisions of the diagram: Horizontal Scale *10.
ViStatus rsrth_ConfigureAcquisitionTime(ViSession instrSession,
                                        ViReal64  acquisitionTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_HORZ_TIME_PER_RECORD, acquisitionTime),
            2, "Acquisition Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the basic settings for the time axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeScale/Sets the horizontal scale - the time per division on the x-axis for
/// HIPAR timeScale/all signals.
/// HIPAR referencePoint/Defines the time reference point in the diagram.
ViStatus rsrth_ConfigureTimeBase(ViSession instrSession,
                                 ViReal64  timeScale,
                                 ViInt32   referencePoint)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_HORZ_SCALE, timeScale),
            2, "Time Scale");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_HORZ_REFERENCE, referencePoint),
            3, "Reference Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the time between two waveform samples.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resolution/Returns the time between two waveform samples.
ViStatus rsrth_QueryResolution(ViSession instrSession,
                               ViReal64* resolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_HORZ_RESOLUTION, resolution),
            2, "Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of recorded analog waveform points
/// HIFN  per second. The sample rate is the reciprocal value of the resolution.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR samplingRateC1C4/Returns the number of recorded analog waveform points per second.
ViStatus rsrth_QueryADCSampleRate(ViSession instrSession,
                                  ViReal64* samplingRateC1C4)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_HORZ_ADC_SAMPLE_RATE, samplingRateC1C4),
            2, "Sampling Rate C1C4");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the record length, the number of recorded
/// HIFN  waveform points that build the waveform across the acquisition time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recordLength/Returns the record length, number of recorded waveform samples.
ViStatus rsrth_QueryRecordLength(ViSession instrSession,
                                 ViInt32*  recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_HORZ_RECORD_LENGTH, recordLength),
            2, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common attributes of the oscilloscope's
/// HIFN  channel subsystem for a particular channel.  These attributes are the
/// HIFN  vertical range, vertical offset, coupling, probe attenuation, and
/// HIFN  channel enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR channelEnabled/Switches the channel signal on or off.
/// HIPAR verticalRange/Sets the voltage range across the 8 vertical divisions of the diagram.
/// HIPAR verticalOffset/Sets the vertical offset in diagram.
/// HIPAR verticalCoupling/Specify how you want the oscilloscope to couple the input signal for
/// HIPAR verticalCoupling/the channel.
ViStatus rsrth_ConfigureChannel(ViSession instrSession,
                                ViInt32   channel,
                                ViBoolean channelEnabled,
                                ViReal64  verticalRange,
                                ViReal64  verticalOffset,
                                ViInt32   verticalCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, repCap, RSRTH_ATTR_CHANNEL_ENABLED, channelEnabled),
            3, "Channel Enabled");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_VERTICAL_RANGE, verticalRange),
            4, "Vertical Range");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_VERTICAL_OFFSET, verticalOffset),
            5, "Vertical Offset");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_VERTICAL_COUPLING, verticalCoupling),
            6, "Vertical Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the offset voltage. The offset is added to
/// HIFN  correct an offset-affected signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR offset/Sets the offset voltage.
ViStatus rsrth_ConfigureChannelOffset(ViSession instrSession,
                                      ViInt32   channel,
                                      ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_OFFSET,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_CHANNEL_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a delay for the selected channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR delay/Sets the channel delay.
ViStatus rsrth_ConfigureChannelDelay(ViSession instrSession,
                                     ViInt32   channel,
                                     ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_DELAY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_CHANNEL_DELAY, delay),
            3, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures  the vertical scale (vertical sensitivity) of
/// HIFN  the indicated channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR verticalScale/Sets the vertical scale for the indicated channel.
ViStatus rsrth_ConfigureChannelVerticalScale(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  verticalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_VERTICAL_SCALE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_VERTICAL_SCALE, verticalScale),
            3, "Vertical Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the bandwidth limit for the indicated
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bandwidth/Selects the bandwidth limit for the indicated channel.
ViStatus rsrth_ConfigureChannelBandwidth(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_BANDWIDTH_LIMIT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_CHANNEL_BANDWIDTH_LIMIT, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the threshold, which is used to obtain the
/// HIFN  signal state. If the signal value is higher than the threshold, the
/// HIFN  signal state is high (1 or true for the Boolean logic). Otherwise, if
/// HIFN  the signal value is below the threshold, the signal state is considered
/// HIFN  low (0 or false). The threshold is used by the pattern and state
/// HIFN  trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR technology/Select a predefined value for one of the most common technologies, or
/// HIPAR technology/select "User" to define an individual threshold.
/// HIPAR userValue/Set an individual threshold value if Threshold is set to User.
ViStatus rsrth_ConfigureChannelThreshold(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   technology,
                                         ViReal64  userValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_THRESHOLD_TECHNOLOGY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_CHANNEL_THRESHOLD_TECHNOLOGY, technology),
            3, "Technology");

    if (technology == RSRTH_VAL_THR_USER)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_CHANNEL_THRESHOLD_USER, userValue),
                4, "User Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function says intrument to set the threshold for the selected
/// HIFN  channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus rsrth_SearchChannelThreshold(ViSession instrSession,
                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_FIND_THRESHOLD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsrth_SetAttributeViString(instrSession, repCap, RSRTH_ATTR_CHANNEL_FIND_THRESHOLD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the threshold value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR threshold/Returns the threshold value.
ViStatus rsrth_QueryChannelThreshold(ViSession instrSession,
                                     ViInt32   channel,
                                     ViReal64* threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CHANNEL_GET_THRESHOLD,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_CHANNEL_GET_THRESHOLD, threshold),
            3, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the attenuation factor of the connected probe. The vertical
/// HIFN  scaling is adjusted accordingly, and measured values are multiplied by
/// HIFN  this factor so that the displayed values are equal to the actual signal
/// HIFN  values.
/// HIFN     Make sure to set the attenuation factor on the instrument according to
/// HIFN  the probe being used. Otherwise, the measurement results do not reflect
/// HIFN  the actual voltage level, and you might misjudge the actual risk.
/// HIFN     If AC coupling is set, the attenuation of passive probes has no
/// HIFN  effect, and voltage is applied to the instrument with factor 1:1.
/// HIFN  Observe the voltage limits, otherwise you can damage the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR probeSettling/Sets the attenuation factor of the connected probe. The vertical
/// HIPAR probeSettling/scaling is adjusted accordingly, and measured values are multiplied by
/// HIPAR probeSettling/this factor so that the displayed values are equal to the actual signal
/// HIPAR probeSettling/values.
/// HIPAR probeFactor/Sets a user-defined attenuation factor if Probe Settling is set to
/// HIPAR probeFactor/User.
/// HIPAR probeUnit/Sets a user-defined probe unit if Probe Settling is set to User.
ViStatus rsrth_ConfigureProbeSettings(ViSession instrSession,
                                      ViInt32   channel,
                                      ViInt32   probeSettling,
                                      ViReal64  probeFactor,
                                      ViInt32   probeUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_PROBE_SETTLING,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_PROBE_SETTLING, probeSettling),
            3, "Probe Settling");

    if (probeSettling == RSRTH_VAL_PROBE_SETTLING_USER)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_PROBE_FACTOR, probeFactor),
                4, "Probe Factor");

        viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_PROBE_UNIT, probeUnit),
                5, "Probe Unit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the inversion of the signal amplitude on or off. To invert means
/// HIFN  to reflect the voltage values of all signal components against the
/// HIFN  horizontal display center. Inversion affects only the display of the
/// HIFN  signal but not the trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR polarity/Turns the inversion of the signal amplitude on or off. To invert means
/// HIPAR polarity/to reflect the voltage values of all signal components against the
/// HIPAR polarity/horizontal display center. Inversion affects only the display of the
/// HIPAR polarity/signal but not the trigger.
ViStatus rsrth_ConfigurePolarityInvertion(ViSession instrSession,
                                          ViInt32   channel,
                                          ViInt32   polarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_INVERT_POLARITY,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_INVERT_POLARITY, polarity),
            3, "Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the trigger type.
/// HIFN     Selects the trigger type, the event type that defines the trigger
/// HIFN  point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerType/Sets the trigger type.
ViStatus rsrth_ConfigureTrigger(ViSession instrSession,
                                ViInt32   triggerType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_TYPE, triggerType),
            2, "Trigger Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode. The trigger mode determines the behavior of the
/// HIFN  instrument if no trigger occurs, and also the number of acquired
/// HIFN  waveforms when a trigger occurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acquisitionMode/Sets the Acquisition mode to single or continuous.
/// HIPAR triggerMode/Sets the trigger mode. The trigger mode determines the behaviour of
/// HIPAR triggerMode/the instrument if no trigger occurs.
ViStatus rsrth_ConfigureTriggerMode(ViSession instrSession,
                                    ViInt32   acquisitionMode,
                                    ViInt32   triggerMode)
{
    ViStatus  error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, acquisitionMode, RSRTH_VAL_ACQ_MODE_SINGLE, RSRTH_VAL_ACQ_MODE_CONTINUOUS),
            2, "Acquisition Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerMode, RSRTH_VAL_NORMAL, RSRTH_VAL_AUTO),
            3, "Trigger Mode");

    if (acquisitionMode == RSRTH_VAL_ACQ_MODE_CONTINUOUS)
    {
        viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_MODE, triggerMode),
                3, "Trigger Mode");
    }
    else
    {
        if (triggerMode == RSRTH_VAL_NORMAL)
        {
            checkErr(RsCore_Write(instrSession, "TRIG:SEAS"));
            checkErr(RsCore_CheckStatus(instrSession, error));
        }
        else
        {
            viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_MODE, RSRTH_VAL_SINGLE),
                    3, "Trigger Mode");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the trigger level voltage.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and channel.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
ViStatus rsrth_ConfigureTriggerChannelLevel(ViSession instrSession,
                                            ViInt32   triggerSource,
                                            ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSRTH_VAL_DATA_C1, RSRTH_VAL_DATA_D7),
            2, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the edge trigger. The trigger occurs when the
/// HIFN  signal
/// HIFN     from the trigger source passes the trigger level in the specified
/// HIFN  direction (slope).
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_TRIGGER_EDGE.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR triggerSlope/Sets the slope for the edge trigger.
ViStatus rsrth_ConfigureEdgeTriggerSource(ViSession instrSession,
                                          ViInt32   triggerSource,
                                          ViReal64  triggerLevel,
                                          ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SLOPE, triggerSlope),
            4, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disbales the noise reject for the edge trigger input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseReject/Enables/disbales the noise reject for the edge trigger input.
ViStatus rsrth_ConfigureEdgeTriggerNoiseReject(ViSession instrSession,
                                               ViBoolean noiseReject)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_TRIGGER_NOISE_REJECT, noiseReject),
            2, "Noise Reject");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameters of edge trigger for external
/// HIFN  source.
/// HIFN     Note(s):
/// HIFN     (1) This function is valid only for R&S RTH1002
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR triggerSlope/Sets the slope for the external edge trigger.
ViStatus rsrth_ConfigureEdgeExternalTrigger(ViSession instrSession,
                                            ViReal64  triggerLevel,
                                            ViInt32   triggerSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_EXTERNAL_TRIGGER_LEVEL, triggerLevel),
            2, "Trigger Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_EXTERNAL_TRIGGER_SLOPE, triggerSlope),
            3, "Trigger Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the runt trigger.  A runt trigger occurs when
/// HIFN  the trigger signal crosses one of the runt thresholds twice without
/// HIFN  crossing the other runt threshold.  You specify the runt thresholds
/// HIFN  with the  Runt Low Threshold and Runt High Threshold parameters.  You
/// HIFN  specify the polarity of the runt with the Runt Polarity parameter.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_RUNT_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR runtLowThreshold/Sets the lower voltage threshold.
/// HIPAR runtHighThreshold/Sets the upper voltage threshold.
/// HIPAR runtPolarity/Defines the polarity of a pulse, that is the direction of the first
/// HIPAR runtPolarity/pulse slope.
ViStatus rsrth_ConfigureRuntTriggerSource(ViSession instrSession,
                                          ViInt32   triggerSource,
                                          ViReal64  runtLowThreshold,
                                          ViReal64  runtHighThreshold,
                                          ViInt32   runtPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSRTH_VAL_DATA_C1, RSRTH_VAL_DATA_D7),
            2, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_RUNT_LOW_THRESHOLD,
            "TriggerChannel", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_RUNT_LOW_THRESHOLD, runtLowThreshold),
            3, "Runt Low Threshold");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_RUNT_HIGH_THRESHOLD, runtHighThreshold),
            4, "Runt High Threshold");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_RUNT_POLARITY, runtPolarity),
            5, "Runt Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Runt trigger range and condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR runtWidth/Defines the upper or lower voltage threshold.
/// HIPAR runtDelta/Sets a range delta t to the specified width.
/// HIPAR runtCondition/Defines the time limit of the runt pulse in relation to the Runt Width
/// HIPAR runtCondition/and Runt Delta settings.
ViStatus rsrth_ConfigureRuntTriggerRange(ViSession instrSession,
                                         ViReal64  runtWidth,
                                         ViReal64  runtDelta,
                                         ViInt32   runtCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_RUNT_CONDITION, runtCondition),
            4, "Runt Condition");

    if (runtCondition == RSRTH_VAL_WITHIN || runtCondition == RSRTH_VAL_OUTSIDE/* || runtCondition == 6*/)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_RUNT_WIDTH_MIN, runtWidth),
                2, "Width Minimum");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_RUNT_WIDTH_MAX, runtDelta),
                3, "Width Maximum");
    }
    else
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_RUNT_WIDTH, runtWidth),
                2, "Runt Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_RUNT_DELTA, runtDelta),
                3, "Runt Delta");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the glitch trigger.  A glitch trigger occurs
/// HIFN  when the trigger signal has a pulse with a width that is less than the
/// HIFN  glitch width.  You specify the glitch width in the Glitch Width
/// HIFN  parameter.  You specify the polarity of the pulse with the Glitch
/// HIFN  Polarity parameter.  The trigger does not actually occur until the edge
/// HIFN  of a pulse that corresponds to the Glitch Width and Glitch Polarity
/// HIFN  crosses the threshold you specify in the Trigger Level parameter.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_GLITCH_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR glitchWidth/Sets the pulse width of the glitch.
/// HIPAR glitchPolarity/Sets the pulse polarity, that is the direction of the first pulse
/// HIPAR glitchPolarity/slope.
/// HIPAR glitchCondition/Selects which glitches are identified: shorter or longer than the
/// HIPAR glitchCondition/width.
ViStatus rsrth_ConfigureGlitchTriggerSource(ViSession instrSession,
                                            ViInt32   triggerSource,
                                            ViReal64  triggerLevel,
                                            ViReal64  glitchWidth,
                                            ViInt32   glitchPolarity,
                                            ViInt32   glitchCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_GLITCH_WIDTH, glitchWidth),
            4, "Glitch Width");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_GLITCH_POLARITY, glitchPolarity),
            5, "Glitch Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_GLITCH_CONDITION, glitchCondition),
            6, "Glitch Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the width trigger.  A width trigger occurs
/// HIFN  when the oscilloscope detects a positive or negative pulse with a width
/// HIFN  between, or optionally outside, the width thresholds.  You specify the
/// HIFN  width thresholds with the Width Low Threshold and Width High Threshold
/// HIFN  parameters.  You specify whether the oscilloscope triggers on pulse
/// HIFN  widths that are within or outside the width thresholds with the Width
/// HIFN  Condition parameter.  You specify the polarity of the pulse with the
/// HIFN  Width Polarity parameter.  The trigger does not actually occur until
/// HIFN  the edge of a pulse that corresponds to the Width Low Threshold, Width
/// HIFN  High Threshold, Width Condition, and Width Polarity crosses the
/// HIFN  threshold you specify in the Trigger Level parameter.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_WIDTH_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR width/For the ranges "Within", "Outside" and "Inside", the width defines the
/// HIPAR width/center of a range which is defined by the limits "+/-Delta". For the
/// HIPAR width/ranges "Shorter" and "Longer", the width defines the maximum and
/// HIPAR width/minimum pulse width, respectively.
/// HIPAR widthDelta/Defines a range around the width value.
/// HIPAR widthPolarity/Sets the polarity of the pulse.
/// HIPAR widthCondition/Defines how the measured pulse width is compared with the given
/// HIPAR widthCondition/limit(s).
ViStatus rsrth_ConfigureWidthTriggerSource(ViSession instrSession,
                                           ViInt32   triggerSource,
                                           ViReal64  triggerLevel,
                                           ViReal64  width,
                                           ViReal64  widthDelta,
                                           ViInt32   widthPolarity,
                                           ViInt32   widthCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSRTH_VAL_CHANNEL1, RSRTH_VAL_D7),
            2, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WIDTH_CONDITION, widthCondition),
            7, "Width Condition");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

    if (widthCondition == RSRTH_VAL_WITHIN || widthCondition == RSRTH_VAL_OUTSIDE)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WIDTH_WIDTH_MIN, width),
                4, "Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WIDTH_WIDTH_MAX, widthDelta),
                5, "Width Delta");
    }
    else
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WIDTH_WIDTH, width),
                4, "Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WIDTH_DELTA, widthDelta),
                5, "Width Delta");
    }

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WIDTH_POLARITY, widthPolarity),
            6, "Width Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the window trigger. The window trigger checks
/// HIFN  the signal run in relation to a "window". The window is formed by the
/// HIFN  upper and lower voltage levels. The event condition is fulfilled, if
/// HIFN  the waveform enters or leaves the window, or if the waveform stays
/// HIFN  inside or outside for a time longer or shorter than specified. With the
/// HIFN  window trigger, you can display longer transient effects.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_WINDOW_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR windowLowerLimit/Sets the lower voltage thresholds for the window trigger.
/// HIPAR windowUpperLimit/Sets the upper voltage limit for the window.
/// HIPAR windowWidth/For the ranges "Within", "Outside" and "Inside", the width defines the
/// HIPAR windowWidth/center of a time range which is defined by the limits "Delta".
/// HIPAR windowDelta/Defines a range around the "Width" value.
/// HIPAR windowCondition/Selects how the signal run is compared with the window.
ViStatus rsrth_ConfigureWindowTriggerSource(ViSession instrSession,
                                            ViInt32   triggerSource,
                                            ViReal64  windowLowerLimit,
                                            ViReal64  windowUpperLimit,
                                            ViReal64  windowWidth,
                                            ViReal64  windowDelta,
                                            ViInt32   windowCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, RSRTH_VAL_CHANNEL1, RSRTH_VAL_D7),
            2, "Trigger Source");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_WINDOW_UPPER_LIMIT,
            "TriggerChannel", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WINDOW_CONDITION, windowCondition),
            7, "Window Condition");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_WINDOW_LOWER_LIMIT, windowLowerLimit),
            3, "Window Lower Limit");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_WINDOW_UPPER_LIMIT, windowUpperLimit),
            4, "Window Upper Limit");

    if (windowCondition == RSRTH_VAL_WITHIN || windowCondition == RSRTH_VAL_OUTSIDE)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WINDOW_WIDTH_MIN, windowWidth),
                5, "Window Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WINDOW_WIDTH_MAX, windowDelta),
                6, "Window Delta");
    }
    else
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WINDOW_WIDTH, windowWidth),
                5, "Window Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_WINDOW_DELTA, windowDelta),
                6, "Window Delta");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects how the time limit of the window is defined.
/// HIFN  Time conditioning is available for the vertical conditions "Stay
/// HIFN  within" and "Stay outside".
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_WINDOW_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowTimeCondition/Defines how the measured pulse width is compared with the given
/// HIPAR windowTimeCondition/limit(s).
ViStatus rsrth_ConfigureWindowTriggerTimeCondition(ViSession instrSession,
                                                   ViInt32   windowTimeCondition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WINDOW_TIME_CONDITION, windowTimeCondition),
            2, "Window Time Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the timeout trigger. The timeout trigger
/// HIFN  event checks if the signal stays above or below the threshold voltage
/// HIFN  for a specified time lapse. In other words, the event occurs if the
/// HIFN  trigger source does not have the expected transition within the
/// HIFN  specified time.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_TIMEOUT_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR timeoutValue/Sets the time limit for the timeout at which the instrument triggers.
/// HIPAR timeoutCondition/Selects the relation of the signal level to the trigger level.
ViStatus rsrth_ConfigureTimeoutTriggerSource(ViSession instrSession,
                                             ViInt32   triggerSource,
                                             ViReal64  triggerLevel,
                                             ViReal64  timeoutValue,
                                             ViInt32   timeoutCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TIMEOUT_VALUE, timeoutValue),
            4, "Timeout Value");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TIMEOUT_CONDITION, timeoutCondition),
            5, "Timeout Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The interval trigger analyzes the time between two pulses. While the
/// HIFN  interval trigger can only analyze either positive or negative polarity,
/// HIFN  searching for an interval is also possible for both polarities at the
/// HIFN  same time ("Either").
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_INTERVAL_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR triggerLevel/Sets the trigger level for the specified event and source.
/// HIPAR intervalWidth/Defines the time between two pulses.
/// HIPAR intervalDelta/Defines a range around the "Interval width" value.
/// HIPAR intervalPolarity/Sets the edge for the trigger. You can analyze the interval between
/// HIPAR intervalPolarity/positive edges or between negative edges.
/// HIPAR intervalCondition/Defines how the time range of an interval is defined.
ViStatus rsrth_ConfigureIntervalTriggerSource(ViSession instrSession,
                                              ViInt32   triggerSource,
                                              ViReal64  triggerLevel,
                                              ViReal64  intervalWidth,
                                              ViReal64  intervalDelta,
                                              ViInt32   intervalPolarity,
                                              ViInt32   intervalCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_INTERVAL_CONDITION, intervalCondition),
            7, "Interval Condition");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

    if (intervalCondition == RSRTH_VAL_WITHIN || intervalCondition == RSRTH_VAL_OUTSIDE)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_INTERVAL_WIDTH_MIN, intervalWidth),
                4, "Width Minimum");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_INTERVAL_WIDTH_MAX, intervalDelta),
                5, "Width Maximum");
    }
    else
    {

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_INTERVAL_WIDTH, intervalWidth),
                4, "Interval Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_INTERVAL_DELTA, intervalDelta),
                5, "Interval Delta");
    }

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_INTERVAL_POLARITY, intervalPolarity),
            6, "Interval Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The slew rate trigger, also known as transition trigger, can detect
/// HIFN  fast or slow edges selectively. It triggers on edges, if the transition
/// HIFN  time from the lower to higher voltage level (or vice versa) is shorter
/// HIFN  or longer as defined, or outside a specified time range. The trigger
/// HIFN  event finds slew rates faster than expected or permissible to avoid
/// HIFN  overshooting and other interfering effects. It also detects very slow
/// HIFN  edges violating the timing in pulse series.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_SLEW_RATE_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIFN     TRIGger:LEVel<n>:SLEW:UPPer
/// HIFN     TRIGger:LEVel<n>:SLEW:LOWer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR slewRateLowerLimit/Defines the lower voltage threshold. When the signal crosses this
/// HIPAR slewRateLowerLimit/level, the slew rate measurement starts or stops depending on the
/// HIPAR slewRateLowerLimit/selected slope.
/// HIPAR slewRateUpperLimit/Defines the upper voltage threshold. When the signal crosses this
/// HIPAR slewRateUpperLimit/level, the slew rate measurement starts or stops depending on the
/// HIPAR slewRateUpperLimit/selected slope.
/// HIPAR slewRatePolarity/Sets the edge direction for the trigger.
/// HIPAR slewRateCondition/Defines the time limits of the slew rate.
ViStatus rsrth_ConfigureSlewRateTriggerSource(ViSession instrSession,
                                              ViInt32   triggerSource,
                                              ViReal64  slewRateLowerLimit,
                                              ViReal64  slewRateUpperLimit,
                                              ViInt32   slewRatePolarity,
                                              ViInt32   slewRateCondition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_SLEW_UPPER_LIMIT,
            "TriggerChannel", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_SLEW_LOWER_LIMIT, slewRateLowerLimit),
            3, "Slew Rate Lower Limit");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_SLEW_UPPER_LIMIT, slewRateUpperLimit),
            4, "Slew Rate Upper Limit");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SLEW_POLARITY, slewRatePolarity),
            5, "Slew Rate Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SLEW_CONDITION, slewRateCondition),
            6, "Slew Rate Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the slew rate range.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_SLEW_RATE_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR slewRate/Sets the slew rate for comparison ranges EQUal, UNEQual, SHORter, and
/// HIPAR slewRate/LONGer.
/// HIPAR slewRateDelta/Sets a range delta t to the specified slew rate.
ViStatus rsrth_ConfigureSlewRateTriggerRange(ViSession instrSession,
                                             ViReal64  slewRate,
                                             ViReal64  slewRateDelta)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  condition = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SLEW_CONDITION, &condition));

    if (condition == 2 || condition == 3)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_SLEW_WIDTH_MIN, slewRate),
                2, "Slew Rate");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_SLEW_WIDTH_MAX, slewRateDelta),
                3, "Slew Rate Delta");
    }
    else
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_SLEW_RATE, slewRate),
                2, "Slew Rate");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_SLEW_DELTA, slewRateDelta),
                3, "Slew Rate Delta");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  With the Data2Clock event - also known as setup/hold - you can analyze
/// HIFN  the relative timing between two signals: a data signal and the
/// HIFN  synchronous clock signal. Many systems require, that the data signal
/// HIFN  must be steady for some time before and after the clock edge, for
/// HIFN  example, the data transmission on parallel interfaces. With this
/// HIFN  trigger type, you can also test the time correlation of sideband and
/// HIFN  inband signals. The event occurs if the data signal crosses the data
/// HIFN  level during the setup and hold time. The reference point for the time
/// HIFN  measurement is defined by clock level and clock edge.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_DATA_TO_CLOCK_TRIGGER.  Call the
/// HIFN  rsrth_ConfigureTrigger function to set the Trigger Type before you call
/// HIFN  this function.
/// HIFN     RSRTH_ATTR_DATA2CLOCK_SOURCE
/// HIFN     RSRTH_ATTR_DATA2CLOCK_HOLD_TIME
/// HIFN     RSRTH_ATTR_DATA2CLOCK_SETUP_TIME
/// HIFN     RSRTH_ATTR_DATA2CLOCK_POLARITY
/// HIFN     RSRTH_ATTR_DATA2CLOCK_DATA_SOURCE
/// HIFN     RSRTH_ATTR_DATA2CLOCK_CONDITION
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIFN     TRIGger:DATatoclock:CSOurce[:VALue]
/// HIFN     TRIGger:DATatoclock:DSOurce[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR dataLevel/Sets the trigger level for the specified event and source.
/// HIPAR clockSource/Selects the input channel of the clock signal.
/// HIPAR data2clockPolarity/Sets the edge of the clock signal.
/// HIPAR dataSource/Selects the input channel of the data signal.
/// HIPAR triggeronSetup/Selects how a violation of the setup and hold time is handled.
/// HIPAR holdTime/Sets the minimum time after the clock edge while the data signal must
/// HIPAR holdTime/stay steady.
/// HIPAR setupTime/Sets the minimum time before the clock edge while the data signal must
/// HIPAR setupTime/stay steady.
ViStatus rsrth_ConfigureData2ClockTriggerSource(ViSession instrSession,
                                                ViInt32   triggerSource,
                                                ViReal64  dataLevel,
                                                ViInt32   clockSource,
                                                ViInt32   data2clockPolarity,
                                                ViInt32   dataSource,
                                                ViInt32   triggeronSetup,
                                                ViReal64  holdTime,
                                                ViReal64  setupTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, dataLevel),
            3, "Data Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_SOURCE, clockSource),
            4, "Clock Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_POLARITY, data2clockPolarity),
            5, "Data2Clock Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_DATA_SOURCE, dataSource),
            6, "Data Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_CONDITION, triggeronSetup),
            7, "Trigger on Setup");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_HOLD_TIME, holdTime),
            8, "Hold Time");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_DATA2CLOCK_SETUP_TIME, setupTime),
            9, "Setup Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the range of a pulse width is defined for keeping up the
/// HIFN  true result of the pattern condition.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_PATTERN_TRIGGER.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIFN     TRIGger:PATTern:TIMeout[:TIME]
/// HIFN     TRIGger:PATTern:WIDTh[:WIDTh]
/// HIFN     TRIGger:PATTern:STATe[:CHANnel<m>]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/Sets a minimum time during which the signals match the pattern
/// HIPAR timeout/definition defined by TRIGger:PATTern:STATe[:CHANnel<m>] and
/// HIPAR timeout/TRIGger:PATTern:STATe:COMBination.
/// HIPAR patternWidth/Sets the width for comparison ranges LONGer, SHORter, EQUal and
/// HIPAR patternWidth/NEQual. For INSide and OUTSide is set Minimum width.
/// HIPAR patternDelta/Sets a range delta t to the pattern width. For INSide and OUTSide is
/// HIPAR patternDelta/set Maximum width.
/// HIPAR patternCondition/Defines how the range of a pulse width is defined for keeping up the
/// HIPAR patternCondition/true result of the pattern condition.
/// HIPAR combination/Sets the logical combination for all active channels.
/// HIPAR state/Sets the state of each input channel.
ViStatus rsrth_ConfigurePatternTriggerSource(ViSession instrSession,
                                             ViReal64  timeout,
                                             ViReal64  patternWidth,
                                             ViReal64  patternDelta,
                                             ViInt32   patternCondition,
                                             ViInt32   combination,
                                             ViInt32   state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PATTERN_CONDITION, patternCondition),
            5, "Pattern Condition");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_PATTERN_TIMEOUT, timeout),
            2, "Timeout");

    if (patternCondition == 2 || patternCondition == 3 || patternCondition == 6)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_PATTERN_WIDTH_MIN, patternWidth),
                3, "Pattern Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_PATTERN_WIDTH_MAX, patternDelta),
                4, "Pattern Delta");
    }
    else
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_PATTERN_WIDTH, patternWidth),
                3, "Pattern Width");

        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_PATTERN_DELTA, patternDelta),
                4, "Pattern Delta");
    }

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PATTERN_COMBINATION, combination),
            6, "Combination");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PATTERN_CHANNEL_STATE, state),
            7, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The serial pattern event is used to trigger on signals with serial
/// HIFN  data patterns in relation to a clock signal - for example, on bus
/// HIFN  signals like the I2C bus. For convenient and comprehensive triggering
/// HIFN  on specific serial data, options for serial protocol analysis are
/// HIFN  provided.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_SERIAL_PATTERN_TRIGGER.  Call the
/// HIFN  rsrth_ConfigureTrigger function to set the Trigger Type before you call
/// HIFN  this function.
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIFN     TRIGger:SPATtern:CSOurce[:VALue]
/// HIFN     TRIGger:SPATtern:DSOurce[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR dataLevel/Sets the trigger level for the specified event and source.
/// HIPAR clockSource/Sets the trigger source for the selected trigger type.
/// HIPAR dataSource/Selects the input channel of the data signal.
/// HIPAR pattern/Defines the serial pattern to be triggered on.
/// HIPAR patternPolarity/Sets the edge at which the data value is sampled.
/// HIPAR firstPolarity/Sets the edge at which the first bit of the pattern is sampled.
/// HIPAR bitOrder/Defines if the data words start with MSBF (most significant bit first)
/// HIPAR bitOrder/or LSBF (least significant bit first).
ViStatus rsrth_ConfigureSerialPatternTriggerSource(ViSession instrSession,
                                                   ViInt32   triggerSource,
                                                   ViReal64  dataLevel,
                                                   ViInt32   clockSource,
                                                   ViInt32   dataSource,
                                                   ViString  pattern,
                                                   ViInt32   patternPolarity,
                                                   ViInt32   firstPolarity,
                                                   ViInt32   bitOrder)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, dataLevel),
            3, "Data Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_SERIAL_PATTERN_SOURCE, clockSource),
            4, "Clock Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SERIAL_PATTERN_DATA_SOURCE, dataSource),
            5, "Data Source");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_SERIAL_PATTERN, pattern),
            6, "Pattern");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SERIAL_PATTERN_POLARITY, patternPolarity),
            7, "Pattern Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SERIAL_PATTERN_FIRST_POLARITY, firstPolarity),
            8, "First Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SERIAL_PATTERN_BIT_ORDER, bitOrder),
            9, "Bit Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the state trigger.
/// HIFN     Notes:
/// HIFN     (1)  This function affects instrument behavior only if the Trigger
/// HIFN  Type is RSRTH_VAL_TRIGGER_STATE.  Call the rsrth_ConfigureTrigger
/// HIFN  function to set the Trigger Type before you call this function.
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIFN     TRIGger:STATe:CHANnel<m>
/// HIFN     TRIGger:STATe:CSOurce[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR dataLevel/Sets the trigger level for the specified event and source.
/// HIPAR state/Sets the state of each input channel.
/// HIPAR clockSource/Sets the trigger source for the selected trigger type.
/// HIPAR combination/Sets the logical combination for all active channels.
/// HIPAR slope/Sets the edge of the clock at which the instrument checks the signal
/// HIPAR slope/states.
ViStatus rsrth_ConfigureStateTrigger(ViSession instrSession,
                                     ViInt32   triggerSource,
                                     ViReal64  dataLevel,
                                     ViInt32   state,
                                     ViInt32   clockSource,
                                     ViInt32   combination,
                                     ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, dataLevel),
            3, "Data Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_STATE_TRIGGER_ENABLED, state),
            4, "State");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_STATE_TRIGGER_CLOCK_SOURCE, clockSource),
            5, "Clock Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_STATE_TRIGGER_COMBINATION, combination),
            6, "Combination");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_STATE_TRIGGER_CLOCK_EDGE, slope),
            7, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the TV trigger.
/// HIFN     Notes:
/// HIFN     (1) This function affects instrument behavior only if the Trigger Type
/// HIFN  is RSRTH_VAL_TV_TRIGGER.  Call the rsrth_ConfigureTrigger function to
/// HIFN  set the Trigger Type before you call this function.
/// HIFN     TRIGger:LEVel<n>[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerSource/Sets the trigger source for the selected trigger type.
/// HIPAR standard/Sets the TV standard.
/// HIPAR mode/Selects the lines or fields on which the instrument triggers.
/// HIPAR mode/Available modes depend on the scanning system of the selected standard.
/// HIPAR lineNumber/Sets the number of the line to be triggered on if TRIGger:TV:MODE is
/// HIPAR lineNumber/LINE.
/// HIPAR lineField/Line field (odd or even) used as reference for counting the video
/// HIPAR lineField/lines. Used by the NTSC standard.
/// HIPAR triggerLevel/Defines the voltage level for the data signal.
/// HIPAR signalPolarity/Sets the polarity of the signal. Note that the sync pulse has the
/// HIPAR signalPolarity/opposite polarity, for example, a positive signal has a negative sync
/// HIPAR signalPolarity/pulse.
ViStatus rsrth_ConfigureTVTrigger(ViSession instrSession,
                                  ViInt32   triggerSource,
                                  ViInt32   standard,
                                  ViInt32   mode,
                                  ViInt32   lineNumber,
                                  ViInt32   lineField,
                                  ViReal64  triggerLevel,
                                  ViInt32   signalPolarity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_TRIGGER_LEVEL,
            "TriggerSourceExt", triggerSource , RS_REPCAP_BUF_SIZE, repCap),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_TRIGGER_LEVEL, triggerLevel),
            7, "Trigger Level");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_POLARITY, signalPolarity),
            8, "Signal Polarity");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_STANDARD, standard),
            3, "Standard");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_MODE, mode),
            4, "Mode");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_LINE_NUMBER, lineNumber),
            5, "Line Number");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_FIELD, lineField),
            6, "Line Field");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the TV trigger custom settings.
/// HIFN     Notes:
/// HIFN     (1) This function affects instrument behavior only if the Trigger Type
/// HIFN  is RSRTH_VAL_TV_TRIGGER.  Call the rsrth_ConfigureTrigger function to
/// HIFN  set the Trigger Type before you call this function.
/// HIFN     (2) Only available if Standard (rsrth_ConfigureTVTrigger) is set to
/// HIFN  Custom.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseType/Sets the type of the sync pulse, either bi-level sync pulse (usually
/// HIPAR pulseType/used in SDTV signals), or tri-level sync pulse (used in HDTV signals).
/// HIPAR linePeriod/Sets the duration of a single video line, the time between two
/// HIPAR linePeriod/successive sync pulses.
/// HIPAR pulseWidth/Sets the width of the sync pulse.
/// HIPAR scanMode/Sets the scanning system.
ViStatus rsrth_ConfigureTVTriggerCustomSettings(ViSession instrSession,
                                                ViInt32   pulseType,
                                                ViReal64  linePeriod,
                                                ViReal64  pulseWidth,
                                                ViInt32   scanMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_PULSE_TYPE, pulseType),
            2, "Pulse Type");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_LINE_PERIOD, linePeriod),
            3, "Line Period");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_PULSE_WIDTH, pulseWidth),
            4, "Pulse Width");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TV_TRIGGER_SCAN_MODE, scanMode),
            5, "Scan Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The "A only" sequence contains an A-event and the holdoff setting as
/// HIFN  optional condition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR holdoffMode/Selects the method to define the holdoff.
/// HIPAR holdoffTime/Sets the time that has to pass at least until the next trigger occurs.
/// HIPAR holdoffTime/The command takes effect if TRIGger:MODE is set to TIME.
/// HIPAR holdoffEvents/Defines the number of triggers to be skipped. The next trigger only
/// HIPAR holdoffEvents/occurs when this number of events is reached. The setting is relevant
/// HIPAR holdoffEvents/if the holdoff mode is set to EVENts.
/// HIPAR randomMinTime/Defines the lower limit for the random time holdoff. The setting is
/// HIPAR randomMinTime/relevant if the holdoff mode is set to RANDom.
/// HIPAR randomMaxTime/Defines the lower limit for the random time holdoff. The setting is
/// HIPAR randomMaxTime/relevant if the holdoff mode is set to RANDom.
ViStatus rsrth_ConfigureAOnlyTriggerSequence(ViSession instrSession,
                                             ViInt32   holdoffMode,
                                             ViReal64  holdoffTime,
                                             ViInt32   holdoffEvents,
                                             ViReal64  randomMinTime,
                                             ViReal64  randomMaxTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SEQUENCE_HOLDOFF_MODE, holdoffMode),
            2, "Holdoff Mode");

    if (holdoffMode != RSRTH_VAL_HOLDOFF_OFF)
    {
        switch (holdoffMode)
        {
        case RSRTH_VAL_HOLDOFF_TIME:
            viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TRIGGER_SEQUENCE_HOLDOFF_TIME, holdoffTime),
                    3, "Holdoff Time");
            break;
        case RSRTH_VAL_HOLDOFF_EVENTS:
            viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_TRIGGER_SEQUENCE_HOLDOFF_EVENTS, holdoffEvents),
                    4, "Holdoff Events");
            break;
        case RSRTH_VAL_HOLDOFF_RANDOM:
            viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TRIGGER_SEQUENCE_HOLDOFF_RANDOM_MINIMUM_TIME, randomMinTime),
                    5, "Random Min Time");

            viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_TRIGGER_SEQUENCE_HOLDOFF_RANDOM_MAXIMUM_TIME, randomMaxTime),
                    6, "Random Max Time");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, holdoffMode), 2, "Holdoff Mode");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR contrastMode/If enabled, the waveforms are displayed in black color on white
/// HIPAR contrastMode/background.
/// HIPAR lcdEnabled/Turns the LCD display on or off.
/// HIPAR touchScreenEnabled/Switches the touch functionality of the screen on or off.
ViStatus rsrth_ConfigureDisplay(ViSession instrSession,
                                ViBoolean contrastMode,
                                ViBoolean lcdEnabled,
                                ViBoolean touchScreenEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_DISPLAY_CONTRAST_MODE, contrastMode),
            2, "Contrast Mode");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_DISPLAY_LCD_STATE, lcdEnabled),
            3, "LCD Enabled");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_DISPLAY_TOUCH_SCREEN_STATE, touchScreenEnabled),
            4, "Touch Screen Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures persistence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines how long every new data point remains on the screen.
/// HIPAR time/Sets a user-defined persistence time. The command takes effect if
/// HIPAR time/DISPlay:PERSistence[:TYPE] is set to TIMe.
ViStatus rsrth_ConfigurePersistence(ViSession instrSession,
                                    ViInt32   type,
                                    ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_DISPLAY_PERSISTENCE_TYPE, type),
            2, "Type");

    if (type == RSRTH_VAL_PERSISTENCE_TIME)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_DISPLAY_PERSISTENCE_TIME, time),
                3, "Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the display zoom.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the zoom.
/// HIPAR scale/Defines the y-value at the beginning of the zoom area.
/// HIPAR position/Sets the center position of the zoomed area in relation to the trigger
/// HIPAR position/point.
ViStatus rsrth_ConfigureZoom(ViSession instrSession,
                             ViBoolean enabled,
                             ViReal64  scale,
                             ViReal64  position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_ZOOM_STATE, enabled),
            2, "Enabled");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_ZOOM_SCALE, scale),
            3, "Scale");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_ZOOM_POSITION, position),
            4, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the history display.
/// HIFN     CHANnel<m>[:WAVeform<n>]:HISTory[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR enableHistory/Enables or disables the history display.
ViStatus rsrth_ConfigureHistoryState(ViSession instrSession,
                                     ViInt32   channel,
                                     ViBoolean enableHistory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_STATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, repCap, RSRTH_ATTR_HISTORY_STATE, enableHistory),
            3, "Enable History");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures start and stop point of history viewer.
/// HIFN     CHANnel<m>:HISTory:STARt
/// HIFN     CHANnel<m>:HISTory:STOP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR start/Sets the index of the first (older) history segment that you want to
/// HIPAR start/see in the history player.
/// HIPAR stop/Sets the index of the last (newer) history segment that you want to
/// HIPAR stop/see in the history player.
ViStatus rsrth_ConfigureHistoryStartStop(ViSession instrSession,
                                         ViInt32   channel,
                                         ViInt32   start,
                                         ViInt32   stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_START_INDEX,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_HISTORY_START_INDEX, start),
            3, "Start");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_HISTORY_STOP_INDEX, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a particular acquisition in the memory to
/// HIFN  display it.
/// HIFN     CHANnel<m>:HISTory:CURRent
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR acquisition/Select the acquisition number in the memory.
ViStatus rsrth_ConfigureHistoryAcquisitionView(ViSession instrSession,
                                               ViInt32   channel,
                                               ViInt32   acquisition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_ACQUISITION_VIEW,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_HISTORY_ACQUISITION_VIEW, acquisition),
            3, "Acquisition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines how fast the history player shows the stored
/// HIFN  waveforms.
/// HIFN     CHANnel<m>:HISTory:TPACq
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR playerSpeed/Defines how fast the history player shows the stored waveforms.
ViStatus rsrth_ConfigureHistoryPlayerSpeed(ViSession instrSession,
                                           ViInt32   channel,
                                           ViInt32   playerSpeed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_PLAYER_SPEED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_HISTORY_PLAYER_SPEED, playerSpeed),
            3, "Player Speed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets if the playback of the selected history segments
/// HIFN  repeats automatically.
/// HIFN     CHANnel<m>:HISTory:REPLay
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR autoRepeat/Sets if the playback of the selected history segments repeats
/// HIPAR autoRepeat/automatically.
ViStatus rsrth_ConfigureHistoryAutoRepeat(ViSession instrSession,
                                          ViInt32   channel,
                                          ViBoolean autoRepeat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_AUTO_REPEAT,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, repCap, RSRTH_ATTR_HISTORY_AUTO_REPEAT, autoRepeat),
            3, "Auto Repeat");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the approximate number of waveforms to be stored.
/// HIFN     CHANnel<m>:HISTory:NSEGments
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR numberofSegments/Sets the approximate number of waveforms to be stored.
ViStatus rsrth_ConfigureHistoryNumberofSegments(ViSession instrSession,
                                                ViInt32   channel,
                                                ViInt32   numberofSegments)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_NUMBER_OF_SEGMENTS,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_HISTORY_NUMBER_OF_SEGMENTS, numberofSegments),
            3, "Number of Segments");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts and stops the replay of the history segments.
/// HIFN     CHANnel<m>:HISTory:PLAY
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus rsrth_HistoryPlaybackControl(ViSession instrSession,
                                      ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_PLAYBACK,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsrth_SetAttributeViString(instrSession, repCap, RSRTH_ATTR_HISTORY_PLAYBACK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the time stamp of the currently displayed
/// HIFN  history acquisition.
/// HIFN     CHANnel<m>:HISTory:TSABsolute?
/// HIFN     CHANnel<m>:HISTory:TSRelative?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR mode/This control selects absolute or relative timestamp:
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the time
/// HIPAR bufferSize/stamp value.
/// HIPAR timeStamp/Returns the time of the currently displayed history acquisition.
ViStatus rsrth_QueryHistoryTimeStamp(ViSession instrSession,
                                     ViInt32   channel,
                                     ViInt32   mode,
                                     ViInt32   bufferSize,
                                     ViChar    timeStamp[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSRTH_VAL_TS_ABS, RSRTH_VAL_TS_REL),
            3, "Mode");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_ABSOLUTE_TIME_STAMP,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    if (mode == RSRTH_VAL_TS_ABS)
    {
        viCheckParm(rsrth_GetAttributeViString(instrSession, repCap, RSRTH_ATTR_HISTORY_ABSOLUTE_TIME_STAMP,
                bufferSize, timeStamp),
                5, "Time Stamp");
    }
    else
    {

        viCheckParm(rsrth_GetAttributeViString(instrSession, repCap, RSRTH_ATTR_HISTORY_RELATIVE_TIME_STAMP,
                bufferSize, timeStamp),
                5, "Time Stamp");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the date of the current acquisition that is shown in the
/// HIFN  history viewer.
/// HIFN     CHANnel<m>:HISTory:TSDat?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the time
/// HIPAR bufferSize/stamp value.
/// HIPAR date/Returns the date of the current acquisition that is shown in the
/// HIPAR date/history viewer.
ViStatus rsrth_QueryHistoryDate(ViSession instrSession,
                                ViInt32   channel,
                                ViInt32   bufferSize,
                                ViChar    date[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_HISTORY_DATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_GetAttributeViString(instrSession, repCap, RSRTH_ATTR_HISTORY_DATE,
            bufferSize, date),
            4, "Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of acquisitions currently saved in
/// HIFN  the memory. This number of acquisitions is available for history
/// HIFN  viewing. It is also the number of acquisitions in an Ultra Segmentation
/// HIFN  acquisition series.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reserved/This control is reserved.
/// HIPAR count/Returns the number of acquisitions currently saved in the memory.
ViStatus rsrth_QueryHistoryAcquisitionsCount(ViSession instrSession,
                                             ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_HISTORY_GET_ACQUISITIONS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the state of the indicated measurement.
/// HIFN     MEASurement<m>[:ENABle]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR measurementState/Enables or disables the measurement.
ViStatus rsrth_ConfigureMeasurementState(ViSession instrSession,
                                         ViInt32   measurement,
                                         ViBoolean measurementState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSRTH_VAL_MEASUREMENT_1, RSRTH_VAL_MEASUREMENT_4),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MEASUREMENT_ENABLED,
            "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, repCap, RSRTH_ATTR_MEASUREMENT_ENABLED, measurementState),
            3, "Measurement State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function disables all active measurements.
/// HIFN     MEASurement<m>:AOFF
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_DisableAllMeasurements(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_MEASUREMENT_DISABLED_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures source channels of the indicated measurement.
/// HIFN     -
/// HIFN     MEASurement<m>:SOURce
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR signalSource/Selects the source 1 of the measurement.
/// HIPAR signalSource2/Selects the source 2 of the measurement.
ViStatus rsrth_ConfigureMeasurementSource(ViSession instrSession,
                                          ViInt32   measurement,
                                          ViInt32   signalSource,
                                          ViInt32   signalSource2)
{
    ViStatus error = VI_SUCCESS;
    ViChar cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString signalSourceArr[] = { "","C1","C2","C3","C4","M1","R","","D0","D1","D2","D3","D4","D5","D6","D7" };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSRTH_VAL_MEASUREMENT_1, RSRTH_VAL_MEASUREMENT_4),
            2, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signalSource, RSRTH_VAL_DATA_C1, RSRTH_VAL_DATA_D7),
            3, "Signal Source");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signalSource2, RSRTH_VAL_DATA_C1, RSRTH_VAL_DATA_D7),
            4, "Signal Source 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS%ld:SOUR %s,%s", measurement, signalSourceArr[signalSource], signalSourceArr[signalSource2]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the indicated measurement.
/// HIFN     MEASurement<m>:DELay:SLOPe
/// HIFN     MEASurement<m>:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR delaySlope/Sets the slope for the delay measurement type.
/// HIPAR type/Selects the measurement type.
ViStatus rsrth_ConfigureMeasurement(ViSession instrSession,
                                    ViInt32   measurement,
                                    ViInt32   delaySlope,
                                    ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSRTH_VAL_MEASUREMENT_1, RSRTH_VAL_MEASUREMENT_4),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MEASUREMENT_DELAY_SLOPE,
            "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MEASUREMENT_DELAY_SLOPE, delaySlope),
            3, "Delay Slope");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MEASUREMENT_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the automatic measurements to be performed on the
/// HIFN  source waveform between the cursor lines.
/// HIFN     CURSor:MEASurement<m>:TYPE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR type/Sets the automatic measurements to be performed on the source waveform
/// HIPAR type/between the cursor lines.
ViStatus rsrth_ConfigureCursorMeasurementType(ViSession instrSession,
                                              ViInt32   measurement,
                                              ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSRTH_VAL_MEASUREMENT_1, RSRTH_VAL_MEASUREMENT_4),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CURSOR_MEASUREMENT,
            "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_CURSOR_MEASUREMENT, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the cursor settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cursorState/Enables or disables the cursor measurement.
/// HIPAR dataSource/Defines the source on which the cursor measurement is performed. The
/// HIPAR dataSource/source setting is not relevant for the vertical cursor type.
/// HIPAR type/Defines the cursor measurement type.
ViStatus rsrth_ConfigureCursor(ViSession instrSession,
                               ViBoolean cursorEnabled,
                               ViInt32   dataSource,
                               ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_CURSOR_ENABLED, cursorEnabled),
            2, "Cursor Enabled");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_CURSOR_SOURCE, dataSource),
            3, "Data Source");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_CURSOR_MEASUREMENT_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the horizontal and vertical position of
/// HIFN  selected cursor or cursor line.
/// HIFN     CURSor:X<CPosition>Position
/// HIFN     CURSor:Y<CPosition>Position
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xPosition1/Defines the position of the left vertical cursor line.
/// HIPAR xPosition2/Defines the position of the left vertical cursor line.
/// HIPAR yPosition1/Set the vertical positions y1 and y2 of the horizontal cursor lines.
/// HIPAR yPosition2/Set the vertical positions y1 and y2 of the horizontal cursor lines.
ViStatus rsrth_ConfigureCursorPosition(ViSession instrSession,
                                       ViReal64  xPosition1,
                                       ViReal64  xPosition2,
                                       ViReal64  yPosition1,
                                       ViReal64  yPosition2)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, "P1", RSRTH_ATTR_CURSOR_X_POSITION, xPosition1),
            2, "X Position 1");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, "P2", RSRTH_ATTR_CURSOR_X_POSITION, xPosition2),
            3, "X Position 2");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, "P1", RSRTH_ATTR_CURSOR_Y_POSITION, yPosition1),
            4, "Y Position 1");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, "P2", RSRTH_ATTR_CURSOR_Y_POSITION, yPosition2),
            5, "Y Position 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function couples the cursor lines so that the distance between
/// HIFN  the two lines remains the same if one cursor is moved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Couples the cursor lines so that the distance between the two lines
/// HIPAR coupling/remains the same if one cursor is moved.
/// HIPAR scaleCoupling/Positions the cursor according scales.
ViStatus rsrth_ConfigureCursorCoupling(ViSession instrSession,
                                       ViBoolean coupling,
                                       ViBoolean scaleCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_CURSOR_COUPLING, coupling),
            2, "Coupling");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_CURSOR_SCALE_COUPLING, scaleCoupling),
            3, "Scale Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the cursors to a default position on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_CursorSetToScreen(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_CURSOR_SET_TO_SCREEN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the record length, number of recorded waveform
/// HIFN  samples.
/// HIFN     ACQuire:POINts[:VALue]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR actualRecordLength/Returns the actual number of points the oscilloscope acquires for each
/// HIPAR actualRecordLength/channel.
ViStatus rsrth_QueryActualRecordLength(ViSession instrSession,
                                       ViInt32*  actualRecordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_HORZ_RECORD_LENGTH, actualRecordLength),
            2, "Actual Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the sample mode the oscilloscope is currently
/// HIFN  using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleMode/Returns the method of adding waveform points to the samples of the ADC
/// HIPAR sampleMode/in order to fill the record length.
ViStatus rsrth_QueryActualSampleMode(ViSession instrSession,
                                     ViInt32*  sampleMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_SAMPLE_MODE, sampleMode),
            2, "Sample Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger type for I2C analysis.
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_MODE
/// HIFN     TRIGger:I2C:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for I2C analysis.
ViStatus rsrth_ConfigureI2CTriggerType(ViSession instrSession,
                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the address trigger conditions.
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ACCESS_BIT
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_CONDITION
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS
/// HIFN     TRIGger<m>:I2C:ACCess
/// HIFN     TRIGger<m>:I2C:ACONdition
/// HIFN     TRIGger<m>:I2C:ADDRess
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rwBitAddress/Toggles the trigger condition between read and write access of the
/// HIPAR rwBitAddress/master. Select 'Either' if the transfer direction is not relevant for
/// HIPAR rwBitAddress/the trigger condition.
/// HIPAR condition/Sets the operator to set a specific address or an address range.
/// HIPAR address/Specifies the address pattern to be found, in binary format. Enter the
/// HIPAR address/pattern in MSB first bit order.
ViStatus rsrth_ConfigureI2CTriggerAddress(ViSession instrSession,
                                          ViInt32   rwBitAddress,
                                          ViInt32   condition,
                                          ViString  address)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ACCESS_BIT, rwBitAddress),
            2, "RW Bit Address");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADDRESS, address),
            4, "Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines how the specified data pattern is compared with
/// HIFN  the acquired signal. The instrument triggers if the acquired data is
/// HIFN  equal or unequal to the pattern.
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_CONDITION
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_PATTERN
/// HIFN     TRIGger:I2C:DCONdition
/// HIFN     TRIGger:I2C:DATA
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal. The instrument triggers if the acquired data is equal or
/// HIPAR condition/unequal to the pattern.
/// HIPAR pattern/Defines the data pattern as trigger condition. Enter the words in MSB
/// HIPAR pattern/first bit order.
ViStatus rsrth_ConfigureI2CTriggerData(ViSession instrSession,
                                       ViInt32   condition,
                                       ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which missing acknowledge bits is detected if
/// HIFN  the trigger type is set to "Missing Ack".
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADRESS_NACK
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_WRITE_NACK
/// HIFN     RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_READ_NACK
/// HIFN     TRIGger:I2C:ADNack
/// HIFN     TRIGger:I2C:DWNack
/// HIFN     TRIGger:I2C:DRNack
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR addressNack/Triggers if the address acknowledge bit is missing - no slave
/// HIPAR addressNack/recognizes the address.
/// HIPAR dataWriteNack/Triggers if a date acknowledge bit is missing - the addressed slave
/// HIPAR dataWriteNack/does not accept the data.
/// HIPAR dataReadNack/Triggers on the end of the read process when the master reads data
/// HIPAR dataReadNack/from the slave. This Nack is sent according to the protocol definition,
/// HIPAR dataReadNack/it is not an error.
ViStatus rsrth_ConfigureI2CTriggerNoAck(ViSession instrSession,
                                        ViBoolean addressNack,
                                        ViBoolean dataWriteNack,
                                        ViBoolean dataReadNack)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_ADRESS_NACK, addressNack),
            2, "Address Nack");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_WRITE_NACK, dataWriteNack),
            3, "Data Write Nack");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_I2C_TRIGGER_DATA_READ_NACK, dataReadNack),
            4, "Data Read Nack");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger type for SPI analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for SPI analysis.
ViStatus rsrth_ConfigureSPITriggerType(ViSession instrSession,
                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SPI_TRIGGER_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure MOSI and MISO trigger conditions.
/// HIFN     Note(s):
/// HIFN     (1) This function has effect only, when trigger type is set to MOSI,
/// HIFN  MISO or MISO/MOSI.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal. The instrument triggers if the acquired data is equal or
/// HIPAR condition/unequal to the pattern.
/// HIPAR position/Sets the number of bits before the first bit of the pattern. These
/// HIPAR position/bits are ignored. The first bit after CS or timeout is bit 0.
/// HIPAR pattern/Specifies the data pattern to be found on the specified line, in
/// HIPAR pattern/binary format. Enter the words in MSB first bit order.
/// HIPAR line/Selects the line, on which the trigger pattern is expected.
ViStatus rsrth_ConfigureSPITriggerData(ViSession instrSession,
                                       ViInt32   condition,
                                       ViInt32   position,
                                       ViString  pattern,
                                       ViInt32   line)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SPI_TRIGGER_DATA_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SPI_TRIGGER_POSITION, position),
            3, "Position");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SPI_TRIGGER_DATA_PATTERN, pattern),
            4, "Pattern");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SPI_TRIGGER_PATTERN_LINE, line),
            5, "Line");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type and source for UART
/// HIFN  analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for UART analysis.
ViStatus rsrth_ConfigureUARTTriggerType(ViSession instrSession,
                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_UART_TRIGGER_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the data conditions if the trigger type is set
/// HIFN  to Data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal.
/// HIPAR pattern/Specifies the data pattern to be found on the specified trigger
/// HIPAR pattern/source, in bin format. Enter the words in msb first bit order.
ViStatus rsrth_ConfigureUARTTriggerData(ViSession instrSession,
                                        ViInt32   condition,
                                        ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_UART_TRIGGER_DATA_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for LIN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for LIN analysis.
ViStatus rsrth_ConfigureLINTriggerType(ViSession instrSession,
                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_MODE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified identifier pattern is compared with the
/// HIPAR condition/acquired signal. The instrument triggers if the acquired address is
/// HIPAR condition/equal or unequal the defined pattern.
/// HIPAR pattern/Specifies the identifier pattern to be found, in binary format. Enter
/// HIPAR pattern/the pattern in MSB first bit order.
ViStatus rsrth_ConfigureLINTriggerID(ViSession instrSession,
                                     ViInt32   condition,
                                     ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_IDENTIFIER_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_IDENTIFIER, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIFN     Note(s):
/// HIFN     (1) This function is available only with RTO-K3 option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal.
/// HIPAR pattern/Defines the data pattern as trigger condition.
ViStatus rsrth_ConfigureLINTriggerData(ViSession instrSession,
                                       ViInt32   condition,
                                       ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_CONDITION_DATA, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_PATTERN_DATA, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIFN     Note(s):
/// HIFN     (1) This function sets trigger type to Error Condition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR checksum/Triggers on a checksum error if TRIGger:LIN:TYPE is set to ERRC.
/// HIPAR idParityError/Triggers on a parity error if TRIGger:LIN:TYPE is set to ERRC.
/// HIPAR syncError/Triggers if a synchronization error occurs.
ViStatus rsrth_ConfigureLINTriggerErrorConditions(ViSession instrSession,
                                                  ViBoolean checksum,
                                                  ViBoolean idParityError,
                                                  ViBoolean syncError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_MODE, RSRTH_VAL_LIN_ERR_COND));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_CHECKSUM_ERROR, checksum),
            2, "Checksum");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_IDENTIFIER_PARITY_ERROR, idParityError),
            3, "ID Parity Error");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_LIN_TRIGGER_SYNC_ERROR, syncError),
            4, "Sync Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger type for CAN analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the trigger type for CAN analysis.
/// HIPAR frameType/Selects the frame type which will be used as trigger condition. For
/// HIPAR frameType/data and remote frames, the identifier format has to be set with
/// HIPAR frameType/TRIGger:CAN:ITYPe.
ViStatus rsrth_ConfigureCANTriggerType(ViSession instrSession,
                                       ViInt32   type,
                                       ViInt32   frameType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_MODE, type),
            2, "Type");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_FRAME_TYPE, frameType),
            3, "Frame Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the CAN trigger FD Bits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fdf/Specifies the CAN FD frame format. It corresponds to the EDL bit
/// HIPAR fdf/(extended data length), which only exists in CAN FD format.
/// HIPAR brs/Sets the bit rate switch bit.
/// HIPAR esi/Sets the error state indicator bit.
ViStatus rsrth_ConfigureCANTriggerFDBits(ViSession instrSession,
                                         ViInt32   fdf,
                                         ViInt32   brs,
                                         ViInt32   esi)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_FDF, fdf),
            2, "FDF");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_BRS, brs),
            3, "BRS");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_FD_BITS_ESI, esi),
            4, "ESI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIFN     Notes(s):
/// HIFN     (1) Trigger type should be set to "Identifier", "Identifier + Data" or
/// HIFN  "Frame Type"
/// HIFN     (2) This function is irrelevant for "Error" and "Overload" frame type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the length of the identifier.
/// HIPAR condition/Defines how the specified identifier pattern is compared with the
/// HIPAR condition/acquired signal.
/// HIPAR pattern/Specifies the identifier pattern to be found, in binary format. Enter
/// HIPAR pattern/the pattern in MSB first bit order.
ViStatus rsrth_ConfigureCANTriggerID(ViSession instrSession,
                                     ViInt32   type,
                                     ViInt32   condition,
                                     ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_ID_TYPE, type),
            2, "Type");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_IDENTIFIER_CONDITION, condition),
            3, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_IDENTIFIER, pattern),
            4, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal.
/// HIPAR pattern/Specifies the data pattern to be found, in binary or hex format.
ViStatus rsrth_ConfigureCANTriggerData(ViSession instrSession,
                                       ViInt32   condition,
                                       ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_DATA_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_PATTERN_DATA, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the byte offset, that defines the start position of the data for
/// HIFN  the data pattern comparison.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR byteOffset/Sets the byte offset, that defines the start position of the data for
/// HIPAR byteOffset/the data pattern comparison.
ViStatus rsrth_ConfigureCANTriggerByteOffset(ViSession instrSession,
                                             ViInt32   byteOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_BYTE_OFFSET, byteOffset),
            2, "Byte Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR crc/Triggers on CRC errors. A CRC error occurs when the CRC calculated by
/// HIPAR crc/the receiver differs from the received value in the CRC sequence.
/// HIPAR stuffing/Triggers if astuff error occurs - when the 6th consecutive equal bit
/// HIPAR stuffing/level in the mentioned fields is detected.
/// HIPAR form/A form error occurs when a fixed-form bit field contains one or more
/// HIPAR form/illegal bits.
/// HIPAR ack/Triggers when the transmitter does not receive an acknowledgment - a
/// HIPAR ack/dominant bit during the Ack Slot.
ViStatus rsrth_ConfigureCANTriggerErrorConditions(ViSession instrSession,
                                                  ViBoolean crc,
                                                  ViBoolean stuffing,
                                                  ViBoolean form,
                                                  ViBoolean ack)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_CHECKSUM_ERROR, crc),
            2, "CRC");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_BIT_STUFFING_ERROR, stuffing),
            3, "Stuffing");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_FORM_ERROR, form),
            4, "Form");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_ACK_ERROR, ack),
            5, "ACK");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers on stuff count errors. A stuff count error occurs if the
/// HIFN  received stuff count value does not match the value calculated from the
/// HIFN  own stuff bit count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stuffCountError/Triggers on stuff count errors. A stuff count error occurs if the
/// HIPAR stuffCountError/received stuff count value does not match the value calculated from the
/// HIPAR stuffCountError/own stuff bit count.
ViStatus rsrth_ConfigureCANTriggerStuffCountError(ViSession instrSession,
                                                  ViBoolean stuffCountError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_CAN_TRIGGER_STUFF_COUNT_ERROR, stuffCountError),
            2, "Stuff Count Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SENT trigger type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the SENT trigger type.
ViStatus rsrth_ConfigureSENTTriggerType(ViSession instrSession,
                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for data trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified data pattern is compared with the acquired
/// HIPAR condition/signal. The instrument triggers if the acquired address is equal or
/// HIPAR condition/unequal the defined pattern.
/// HIPAR pattern/Defines the data pattern as trigger condition. Enter the words in MSB
/// HIPAR pattern/first bit order.
ViStatus rsrth_ConfigureSENTTriggerData(ViSession instrSession,
                                        ViInt32   condition,
                                        ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_DATA_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_DATA_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for ID trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified identifier pattern is compared with the
/// HIPAR condition/acquired signal. The instrument triggers if the acquired address is
/// HIPAR condition/equal or unequal the defined pattern.
/// HIPAR pattern/Specifies the identifier pattern to be found, in binary format. Enter
/// HIPAR pattern/the pattern in MSB first bit order.
ViStatus rsrth_ConfigureSENTTriggerID(ViSession instrSession,
                                      ViInt32   condition,
                                      ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_IDENTIFIER_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_IDENTIFIER_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Status trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR condition/Defines how the specified status pattern is compared with the acquired
/// HIPAR condition/signal. The instrument triggers if the acquired address is equal or
/// HIPAR condition/unequal the defined pattern.
/// HIPAR pattern/Defines the status pattern as trigger condition. Enter the words in
/// HIPAR pattern/MSB first bit order.
ViStatus rsrth_ConfigureSENTTriggerStatus(ViSession instrSession,
                                          ViInt32   condition,
                                          ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_STATUS_CONDITION, condition),
            2, "Condition");

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_STATUS_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures conditions for Error Condition trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastCRCError/Triggers on a detected CRC error in the transmission sequences of the
/// HIPAR fastCRCError/fast channel. The CRC length is 4 bits.
/// HIPAR slowCRCError/Triggers on a detected CRC error in serial messages of the slow
/// HIPAR slowCRCError/channel. The CRC length is 4 bits for short serial messages, and 6 bit
/// HIPAR slowCRCError/of enhanced serial messages.
/// HIPAR frameLengthError/Triggers on frame length errors in transmission sequences. A frame
/// HIPAR frameLengthError/length error occurs, when the total length of the transmission
/// HIPAR pulsePeriodError/Triggers on an error in the calibration/sync pulse in transmission
/// HIPAR pulsePeriodError/sequences of the fast channel.
/// HIPAR syncPulseError/Trigggers on a synchronization/calibration pulse error in transmission
/// HIPAR syncPulseError/sequences of the fast channel.
ViStatus rsrth_ConfigureSENTTriggerErrorConditions(ViSession instrSession,
                                                   ViBoolean fastCRCError,
                                                   ViBoolean slowCRCError,
                                                   ViBoolean frameLengthError,
                                                   ViBoolean pulsePeriodError,
                                                   ViBoolean syncPulseError)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_FAST_CRC_ERROR, fastCRCError),
            2, "Fast CRC Error");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_SLOW_CRC_ERROR, slowCRCError),
            3, "Slow CRC Error");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_FRAME_LENGTH_ERROR, frameLengthError),
            4, "Frame Length Error");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_PULSE_PERIOD_ERROR, pulsePeriodError),
            5, "Pulse Period Error");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROTOCOL_SENT_TRIGGER_SYNC_PULSE_ERROR, syncPulseError),
            6, "Sync Pulse Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the filename of the next screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the file name when an image of the display is stored to a file
/// HIPAR name/rather than printed to a printer using the HCOPy:IMMediate command.
ViStatus rsrth_HardcopySetFileName(ViSession instrSession,
                                   ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_HARDCOPY_FILE_NAME, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the current display in a new screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_HardcopyPrint(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_HARDCOPY_PRINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a destination file for the hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR color/Defines the color mode for printing.
/// HIPAR destination/Defines the format of the screenshot file.
ViStatus rsrth_ConfigureHardcopyDevice(ViSession instrSession,
                                       ViBoolean bwColor,
                                       ViInt32   destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_HARDCOPY_BWCOLOR, bwColor),
            2, "BW Color");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_HARDCOPY_LANGUAGE_OUTPUT_FORMAT, destination),
            3, "Destination");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Inverts the colors of the output, i.e. a dark waveform is printed on a
/// HIFN  white background.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hardcopyInverse/Inverts the colors of the output, i.e. a dark waveform is printed on a
/// HIPAR hardcopyInverse/white background.
ViStatus rsrth_ConfigureHardcopyInverse(ViSession instrSession,
                                        ViBoolean hardcopyInverse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_HARDCOPY_INVERSE, hardcopyInverse),
            2, "Hardcopy Inverse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the logical analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the logic channels.
/// HIPAR coupling/Couples the threshold and hysteresis settings for the logic channels.
ViStatus rsrth_ConfigureLogicAnalyzer(ViSession instrSession,
                                      ViBoolean enabled,
                                      ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_LOGIC_CHANNEL_STATE, enabled),
            2, "Enabled");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_LOGIC_CHANNEL_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the deskew for all channels of a logic probe at
/// HIFN  once, or for each logic channel separately.
/// HIFN     LOGic:CHANnel<m>:DESKew
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Sets the logical channel number.
/// HIPAR deskew/Sets the deskew for all channels of a logic probe at once, or for each
/// HIPAR deskew/logic channel separately.
ViStatus rsrth_ConfigureLogicAnalyzerChannel(ViSession instrSession,
                                             ViInt32   channel,
                                             ViReal64  deskew)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 9),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_LOGIC_CHANNEL_DESKEW,
            "LogicChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_LOGIC_CHANNEL_DESKEW, deskew),
            3, "Deskew");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the logic channel group.
/// HIFN     LOGic:GROup<m>:TECHnology
/// HIFN     LOGic:GROup<m>:USER
/// HIFN     LOGic:GROup<m>:HYSTeresis
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR group/Sets the logic channel group.
/// HIPAR threshold/Sets the threshold value for the selected channel group, or for all
/// HIPAR threshold/logic channels.
/// HIPAR userThreshold/Sets the threshold value if LOGic:GROup<m>:TECHnology is set to USER.
/// HIPAR hysteresis/Sets the logical channel group hysteresis.
ViStatus rsrth_ConfigureLogicGroup(ViSession instrSession,
                                   ViInt32   group,
                                   ViInt32   threshold,
                                   ViReal64  userThreshold,
                                   ViInt32   hysteresis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, group, 1, 3),
            2, "Group");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_THRESHOLD_TECHNOLOGY,
            "LCHGroup", group - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Group");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_THRESHOLD_TECHNOLOGY, threshold),
            3, "Threshold");

    if (threshold == RSRTH_VAL_THR_USER)
    {
        viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_THRESHOLD, userThreshold),
                4, "User Threshold");
    }

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_HYSTERESIS, hysteresis),
            5, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the current threshold value.
/// HIFN     LOGic:GROup<m>:THReshold?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR group/Sets the logic channel group.
/// HIPAR threshold/Returns the current threshold value.
ViStatus rsrth_QueryLogicGroupThreshold(ViSession instrSession,
                                        ViInt32   group,
                                        ViReal64* threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, group, 1, 3),
            2, "Group");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_GET_THRESHOLD,
            "LCHGroup", group - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Group");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_LOGIC_CHANNEL_GROUP_GET_THRESHOLD, threshold),
            3, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the connection state of the logic probe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeState/Returns the connection state of the logic probe.
ViStatus rsrth_QueryLogicProbeState(ViSession  instrSession,
                                    ViBoolean* probeState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_PROBE_STATE, probeState),
            2, "Probe State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures mask definitions.
/// HIFN     MASK:CHANnel<m>:STATe
/// HIFN     MASK:CHANnel<m>:PROPerties:XWIDth
/// HIFN     MASK:CHANnel<m>:PROPerties:YWIDth
/// HIFN     MASK:ONViolation[:SELection]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR enabled/Turns the selected mask on or off.
/// HIPAR widthX/Changes the width of the selected mask in horizontal direction.
/// HIPAR widthY/Sets the width of the selected mask in vertical direction.
/// HIPAR actiononViolation/Defines the action to be executed if a violation occurs.
ViStatus rsrth_ConfigureMaskTest(ViSession instrSession,
                                 ViInt32   channel,
                                 ViBoolean enabled,
                                 ViReal64  widthX,
                                 ViReal64  widthY,
                                 ViInt32   actiononViolation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MASK_TEST_ENABLED,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, repCap, RSRTH_ATTR_MASK_TEST_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_MASK_TEST_HORIZONTAL_WIDTH, widthX),
            4, "Width X");

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_MASK_TEST_VERTICAL_WIDTH, widthY),
            5, "Width Y");

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MASK_TEST_VIOLATION_EVENT, actiononViolation),
            6, "Action on Violation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a mask from the envelope waveform of the
/// HIFN  selected waveform with the defined width in x and y direction.
/// HIFN     MASK:CHANnel<m>:CREatemask
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
ViStatus rsrth_CreateMaskTest(ViSession instrSession,
                              ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MASK_TEST_CREATE,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    checkErr(rsrth_SetAttributeViString(instrSession, repCap, RSRTH_ATTR_MASK_TEST_CREATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent IVI attributes.  If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes.  It is best to use the high-level driver
/// HIFN  functions as much as possible.  They handle order dependencies and
/// HIFN  multithread locking for you.  In addition, they perform status checking
/// HIFN  only after setting all of the attributes.  In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change.  Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus rsrth_SetAttributeViInt32(ViSession instrSession,
                                   ViString  channelName,
                                   ViAttr    attributeId,
                                   ViInt32   value)
{
    return RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent IVI attributes.  If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes.  It is best to use the high-level driver
/// HIFN  functions as much as possible.  They handle order dependencies and
/// HIFN  multithread locking for you.  In addition, they perform status checking
/// HIFN  only after setting all of the attributes.  In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change.  Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus rsrth_SetAttributeViInt64(ViSession instrSession,
                                   ViString  channelName,
                                   ViAttr    attributeId,
                                   ViInt64   value)
{
    return RsCore_SetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent IVI attributes.  If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes.  It is best to use the high-level driver
/// HIFN  functions as much as possible.  They handle order dependencies and
/// HIFN  multithread locking for you.  In addition, they perform status checking
/// HIFN  only after setting all of the attributes.  In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change.  Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus rsrth_SetAttributeViReal64(ViSession instrSession,
                                    ViString  channelName,
                                    ViAttr    attributeId,
                                    ViReal64  value)
{
    return RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a Vi String attribute. It is also used
/// HIFN  to set an event-type attribute (attribute without a parameter). In this
/// HIFN  case the string value is ignored.
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent IVI attributes.  If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes.  It is best to use the high-level driver
/// HIFN  functions as much as possible.  They handle order dependencies and
/// HIFN  multithread locking for you.  In addition, they perform status checking
/// HIFN  only after setting all of the attributes.  In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change.  Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus rsrth_SetAttributeViString(ViSession instrSession,
                                    ViString  channelName,
                                    ViAttr    attributeId,
                                    ViString  value)
{
    return RsCore_SetAttributeViStringOrRawString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent IVI attributes.  If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes.  It is best to use the high-level driver
/// HIFN  functions as much as possible.  They handle order dependencies and
/// HIFN  multithread locking for you.  In addition, they perform status checking
/// HIFN  only after setting all of the attributes.  In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change.  Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus rsrth_SetAttributeViBoolean(ViSession instrSession,
                                     ViString  channelName,
                                     ViAttr    attributeId,
                                     ViBoolean value)
{
    return RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent IVI attributes.  If the attribute represents an
/// HIFN  instrument state, this function performs instrument I/O in the
/// HIFN  following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViInt32 variable.
ViStatus rsrth_GetAttributeViInt32(ViSession instrSession,
                                   ViString  channelName,
                                   ViAttr    attributeId,
                                   ViInt32*  value)
{
    return RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent IVI attributes.  If the attribute represents an
/// HIFN  instrument state, this function performs instrument I/O in the
/// HIFN  following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViInt32 variable.
ViStatus rsrth_GetAttributeViInt64(ViSession instrSession,
                                   ViString  channelName,
                                   ViAttr    attributeId,
                                   ViInt64*  value)
{
    return RsCore_GetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent IVI attributes.  If the attribute represents an
/// HIFN  instrument state, this function performs instrument I/O in the
/// HIFN  following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViReal64 variable.
ViStatus rsrth_GetAttributeViReal64(ViSession instrSession,
                                    ViString  channelName,
                                    ViAttr    attributeId,
                                    ViReal64* value)
{
    return RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent IVI attributes.  If the attribute represents an
/// HIFN  instrument state, this function performs instrument I/O in the
/// HIFN  following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid.
/// HIFN     You must provide a ViChar array to serve as a buffer for the value.
/// HIFN  You pass the number of bytes in the buffer as the Buffer Size
/// HIFN  parameter.  If the current value of the attribute, including the
/// HIFN  terminating NUL byte, is larger than the size you indicate in the
/// HIFN  Buffer Size parameter, the function copies Buffer Size - 1 bytes into
/// HIFN  the buffer, places an ASCII NUL byte at the end of the buffer, and
/// HIFN  returns the buffer size you must pass to get the entire value.  For
/// HIFN  example, if the value is "123456" and the Buffer Size is 4, the
/// HIFN  function places "123" into the buffer and returns 7.
/// HIFN     If you want to call this function just to get the required buffer
/// HIFN  size, you can pass 0 for the Buffer Size and NULL for the Attribute
/// HIFN  Value buffer.
/// HIFN     If you want the function to fill in the buffer regardless of the
/// HIFN  number of bytes in the value, pass a negative number for the Buffer
/// HIFN  Size parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
ViStatus rsrth_GetAttributeViString(ViSession instrSession,
                                    ViString  channelName,
                                    ViAttr    attributeId,
                                    ViInt32   bufferSize,
                                    ViChar    value[])
{
    return RsCore_GetAttributeViStringOrRawString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize, value);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent IVI attributes.  If the attribute represents an
/// HIFN  instrument state, this function performs instrument I/O in the
/// HIFN  following cases:
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN     - State caching is enabled and the currently cached value is invalid.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViBoolean variable.
ViStatus rsrth_GetAttributeViBoolean(ViSession  instrSession,
                                     ViString   channelName,
                                     ViAttr     attributeId,
                                     ViBoolean* value)
{
    return RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  Function used by Hi-level functions to obtain a repCap name from:
/// HIFN         - attributeID
/// HIFN         - repCapNameID
/// HIFN         - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repcapNameID/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the
/// HIPAR bufferSize/repCapName.
/// HIPAR repcapname/String returning RepCap name.
ViStatus rsrth_GetAttributeRepCapName(ViSession instrSession,
                                      ViUInt32  attributeID,
                                      ViString  repcapNameID,
                                      ViInt32   index,
                                      ViInt32   bufferSize,
                                      ViChar    repcapname[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeID, repcapNameID, index, bufferSize, repcapname));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the repeated capability id(s) that belongs to
/// HIFN  the attribute you specify. More then one id is listed in comma
/// HIFN  separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityId_s/The buffer in which the function returns the repeated capability id(s)
/// HIPAR repeatedCapabilityId_s/belonging to the attribute. The buffer must be of type ViChar and have
/// HIPAR repeatedCapabilityId_s/at least as many bytes as indicated in the Buffer Size parameter.
ViStatus rsrth_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/// HIFN  This function returns the repeated capability name(s) that belongs to
/// HIFN  the attribute and single repeated capability id you may specify. More
/// HIFN  then one name is listed in comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityId/this parameter defines the Id of the repeated capability on which to
/// HIPAR repeatedCapabilityId/query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id
/// HIPAR repeatedCapabilityIdNames/name(s) belonging to the attribute. The buffer must be of type ViChar
/// HIPAR repeatedCapabilityIdNames/and have at least as many bytes as indicated in the Buffer Size
/// HIPAR repeatedCapabilityIdNames/parameter.
ViStatus rsrth_GetAttributeRepeatedCapabilityIdNames(ViSession instrSession,
                                                     ViUInt32  attributeID,
                                                     ViString  repeatedCapabilityId,
                                                     ViInt32   bufferSize,
                                                     ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN  Enables or disables the math channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathEnabled/Enables or disables the math channel.
ViStatus rsrth_ConfigureMathEnabled(ViSession instrSession,
                                    ViBoolean mathEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_MATH_WAVEFORM_ENABLED, mathEnabled),
            2, "Math Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the operation to calculate a math waveform.
/// HIFN     CALCulate:MATH[:EXPRession][:DEFine]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR expression/Sets the operation to calculate a math waveform.
ViStatus rsrth_ConfigureMathExpression(ViSession instrSession,
                                       ViString  expression)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_MATH_EXPRESSION, expression),
            2, "Expression");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the voltage range across the 8 vertical divisions of the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verticalRange/Sets the voltage range across the 8 vertical divisions of the diagram.
ViStatus rsrth_ConfigureMathVerticalRange(ViSession instrSession,
                                          ViReal64  verticalRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_MATH_VERTICAL_RANGE, verticalRange),
            2, "Vertical Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the vertical scale (vertical sensitivity) of the math waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verticalScale/Sets the vertical scale (vertical sensitivity) of the math waveform.
ViStatus rsrth_ConfigureMathVerticalScale(ViSession instrSession,
                                          ViReal64  verticalScale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_MATH_VERTICAL_SCALE, verticalScale),
            2, "Vertical Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves the math waveform or down in the diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mathNumber/Selects the math waveform.
/// HIPAR verticalPosition/Moves the math waveform or down in the diagram.
ViStatus rsrth_ConfigureMathVerticalPosition(ViSession instrSession,
                                             ViReal64  verticalPosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_MATH_VERTICAL_POSITION, verticalPosition),
            2, "Vertical Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restarts the envelope and average calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_MathReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_MATH_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the waveform to be taken as reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the waveform to be taken as reference waveform.
ViStatus rsrth_ConfigureReferenceWaveformSource(ViSession instrSession,
                                                ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates the reference waveform from the source waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_UpdateReferenceWaveformWithSelectedSource(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayEnabled/Activates or deactivates the reference waveform.
ViStatus rsrth_ConfigureReferenceWaveformDisplayState(ViSession instrSession,
                                                      ViBoolean displayEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_DISPLAY_ENABLED, displayEnabled),
            2, "Display Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the math waveform display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformName/Defines the name of the reference waveform file to be loaded, saved or
/// HIPAR waveformName/deleted.
/// HIPAR operation/This control selects the type of waveform operation to be performed.
ViStatus rsrth_ReferenceWaveformFileOperations(ViSession instrSession,
                                               ViString  waveformName,
                                               ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSRTH_VAL_WAV_SELECT, RSRTH_VAL_WAV_DELETE),
            2, "Operation");

    switch (operation)
    {
    case RSRTH_VAL_WAV_SELECT:
        viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_NAME, waveformName),
                3, "Waveform Name");
        break;

    case RSRTH_VAL_WAV_OPEN:
        checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_LOAD, NULL));
        break;

    case RSRTH_VAL_WAV_SAVE:
        checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_SAVE, NULL));
        break;

    case RSRTH_VAL_WAV_DELETE:
        checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_DELETE, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 4, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the vertical position of the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR verticalPosition/Sets the vertical position of the reference waveform.
ViStatus rsrth_ConfigureReferenceWaveformVerticalScaling(ViSession instrSession,
                                                         ViReal64  verticalPosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_REFERENCE_WAVEFORM_VERTICAL_POSITION, verticalPosition),
            2, "Vertical Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the header of channel waveform data. The header contains setup
/// HIFN  and acquisition settings.
/// HIFN     CHANnel<m>:DATA:HEADer?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR waveformHeader/Returns the header of channel waveform data. The header contains setup
/// HIPAR waveformHeader/and acquisition settings.
ViStatus rsrth_FetchWaveformHeader(ViSession instrSession,
                                   ViInt32   channel,
                                   ViReal64* xStart,
                                   ViReal64* xStop,
                                   ViInt32*  recordLength,
                                   ViInt32*  valuesPerSample)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_WAVEFORM_HEADER,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_GetAttributeViString(instrSession, repCap, RSRTH_ATTR_WAVEFORM_HEADER, RS_MAX_MESSAGE_BUF_SIZE, response),
            4, "Waveform Header");

    checkErr(RsCore_GetViReal64AtCsvIndex(response, 0, xStart));
    checkErr(RsCore_GetViReal64AtCsvIndex(response, 1, xStop));
    checkErr(RsCore_GetViInt32AtCsvIndex(response, 2, recordLength));
    checkErr(RsCore_GetViInt32AtCsvIndex(response, 3, valuesPerSample));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Waveform object with data of a particular size needed only for reusing
/// HIFN  waveform object across reads.
/// HIFN     -
/// HIFN     CHANnel<m>:DATA[:VALues]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR waveform/Waveform object with data of a particular size needed only for reusing
/// HIPAR waveform/waveform object across reads.
ViStatus rsrth_FetchWaveform(ViSession instrSession,
                             ViInt32   channel,
                             ViInt32   bufferSize,
                             ViChar    waveform[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CHAN%ld:DATA?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, waveform, NULL));
    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method initiates an acquisition on the channels that you enable
/// HIFN  with the Configure Channel function.  It then waits for the acquisition
/// HIFN  to complete, and returns the data of the channel waveform points for
/// HIFN  transmission from the instrument to the controlling computer.
/// HIFN     -
/// HIFN     CHANnel:DATA[:VALues]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Defines the size of buffer passed to 'Waveform Header'.
/// HIPAR waveform/Waveform object with data of a particular size needed only for reusing
/// HIPAR waveform/waveform object across reads.
ViStatus rsrth_ReadWaveform(ViSession instrSession,
                            ViInt32   channel,
                            ViInt32   bufferSize,
                            ViChar    waveform[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CHAN%ld:DATA?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, waveform, NULL));
    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the continuous acquisition.
/// HIFN     Notes:
/// HIFN     (1) This function does not check the instrument status.   Typically,
/// HIFN  you call this function only in a sequence of calls to other low-level
/// HIFN  driver functions.  The sequence performs one operation.  You use the
/// HIFN  low-level functions to optimize one or more aspects of interaction with
/// HIFN  the instrument.  If you want to check the instrument status, call the
/// HIFN  rsrth_error_query function at the conclusion of the sequence.
/// HIFN     RUN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_RunWithoutWait(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_RUN_NO_WAIT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the continuous acquisition.
/// HIFN     Notes:
/// HIFN     (1) This function does not check the instrument status.   Typically,
/// HIFN  you call this function only in a sequence of calls to other low-level
/// HIFN  driver functions.  The sequence performs one operation.  You use the
/// HIFN  low-level functions to optimize one or more aspects of interaction with
/// HIFN  the instrument.  If you want to check the instrument status, call the
/// HIFN  rsrth_error_query function at the conclusion of the sequence.
/// HIFN     RUN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_RunAndWait(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_RUN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stops the running acquistion.
/// HIFN     Notes:
/// HIFN     (1) This function does not check the instrument status.   Typically,
/// HIFN  you call this function only in a sequence of calls to other low-level
/// HIFN  driver functions.  The sequence performs one operation.  You use the
/// HIFN  low-level functions to optimize one or more aspects of interaction with
/// HIFN  the instrument.  If you want to check the instrument status, call the
/// HIFN  rsrth_error_query function at the conclusion of the sequence.
/// HIFN     STOP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_Stop(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries measurement results.
/// HIFN     MEASurement<m>:RESult:ACTual?
/// HIFN     MEASurement<m>:RESult:LIMit?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR result/Returns the result of the indicated measurement.
/// HIPAR limit/Indicates whether the measurement results are inside the measurement
/// HIPAR limit/range, or outside (clipping).
ViStatus rsrth_QueryMeasurementsResults(ViSession instrSession,
                                        ViInt32   measurement,
                                        ViReal64* result,
                                        ViInt32*  limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSRTH_VAL_MEASUREMENT_1, RSRTH_VAL_MEASUREMENT_4),
            2, "Measurement");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MEASUREMENT_RESULT,
            "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_MEASUREMENT_RESULT, result),
            3, "Result");

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_MEASUREMENT_RESULT_LIMIT, limit),
            4, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns Mask Test measurement results.
/// HIFN     MASK:CHANnel<m>:RESult:TOTL[:COUNt]?
/// HIFN     MASK:CHANnel<m>:RESult:PASS[:COUNt]?
/// HIFN     MASK:CHANnel<m>:RESult:PASS:PERCentage?
/// HIFN     MASK:CHANnel<m>:RESult:FAIL[:COUNt]?
/// HIFN     MASK:CHANnel<m>:RESult:FAIL:PERCentage?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR total/Returns the number of tested acquisitions.
/// HIPAR passed/Returns the number of passed acquisitions.
/// HIPAR passedPct/Returns the percentage share of passed acquisitions.
/// HIPAR failed/Returns the number of failed acquisitions.
/// HIPAR failedPct/Returns the percentage share of failed acquisitions.
ViStatus rsrth_QueryMaskTestResults(ViSession instrSession,
                                    ViInt32   channel,
                                    ViInt32*  total,
                                    ViInt32*  passed,
                                    ViReal64* passedPct,
                                    ViInt32*  failed,
                                    ViReal64* failedPct)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, RSRTH_VAL_CHANNEL_1, RSRTH_VAL_CHANNEL_4),
            2, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_MASK_TEST_RESULT_TOTAL,
            "Channel", channel - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Channel");

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MASK_TEST_RESULT_TOTAL, total),
            3, "Total");

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MASK_TEST_RESULT_PASSED, passed),
            4, "Passed");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_MASK_TEST_RESULT_PASSED_PCT, passedPct),
            5, "Passed Pct");

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_MASK_TEST_RESULT_FAILED, failed),
            6, "Failed");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_MASK_TEST_RESULT_FAILED_PCT, failedPct),
            7, "Failed Pct");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns Mask Test elapsed times.
/// HIFN     -
/// HIFN     MASK:ELAPsedtime[:SECS]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR duration/Returns the test duration in seconds.
/// HIPAR days/Test time in days.
/// HIPAR hours/Test time in hours.
/// HIPAR minutes/Test time in minutes.
/// HIPAR seconds/Test time in seconds.
/// HIPAR deciseconds/Test time in deciseconds.
ViStatus rsrth_QueryMaskTestElapsedTime(ViSession instrSession,
                                        ViInt32*  duration,
                                        ViInt32*  days,
                                        ViInt32*  hours,
                                        ViInt32*  minutes,
                                        ViInt32*  seconds,
                                        ViInt32*  deciseconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  response = NULL;
    ViChar* p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_MASK_TEST_DURATION, duration),
            2, "Duration");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "MASK:ELAP:TOT?", &response));

    p2buf = strtok(response, ";");
    *days = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ";");
    *hours = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ";");
    *minutes = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ";");
    *seconds = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok(NULL, ";");
    *deciseconds = RsCore_Convert_String_To_ViInt32(p2buf);

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the state of the mask test.
/// HIFN     MASK[:TESTstate]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Returns the state of the mask test.
ViStatus rsrth_QueryMaskTestState(ViSession instrSession,
                                  ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_MASK_TEST_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the counters of passed and failed acquisitions to
/// HIFN  Zero.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_ResetAllMasksResults(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_MASK_TEST_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns Cursor results.
/// HIFN     CURSor:MEASurement<m>:RESult:ACTual?
/// HIFN     CURSor:MEASurement<m>:RESult:LIMit?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
/// HIPAR result/Returns the result of the indicated cursor measurement.
/// HIPAR limit/Indicates whether the measurement results are inside the measurement
/// HIPAR limit/range, or outside (clipping).
ViStatus rsrth_QueryCursorResults(ViSession instrSession,
                                  ViInt32   measurement,
                                  ViReal64* result,
                                  ViInt32*  limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRTH_ATTR_CURSOR_MEASUREMENT_RESULT,
            "Measurement", measurement - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Measurement");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, repCap, RSRTH_ATTR_CURSOR_MEASUREMENT_RESULT, result),
            3, "Result");

    viCheckParm(rsrth_GetAttributeViInt32(instrSession, repCap, RSRTH_ATTR_CURSOR_MEASUREMENT_RESULT_LIMIT, limit),
            4, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns Cursor Distances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR horizontal/Returns the absolute value of the difference between the positions of
/// HIPAR horizontal/horizontal cursor lines y.
/// HIPAR vertical/Returns the time difference  between to vertical cursor lines.
/// HIPAR inverse/Returns the inverse value of time difference between to vertical
/// HIPAR inverse/cursor lines1/t.
ViStatus rsrth_QueryCursorDistances(ViSession instrSession,
                                    ViReal64* horizontal,
                                    ViReal64* vertical,
                                    ViReal64* inverse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_CURSOR_HORIZONTAL_DISTANCE, horizontal),
            2, "Horizontal");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_CURSOR_VERTICAL_DISTANCE, vertical),
            3, "Vertical");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_CURSOR_INVERSE_DISTANCE, inverse),
            4, "Inverse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns Cursor Amplitudes.
/// HIFN     RSRTH_ATTR_CURSOR_MEASUREMENT_AMPLITUDE_1
/// HIFN     RSRTH_ATTR_CURSOR_MEASUREMENT_AMPLITUDE_2
/// HIFN     CURSor:Y1AMplitude?
/// HIFN     CURSor:Y2AMplitude?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitude1/Returns amplitude 1.
/// HIPAR amplitude2/Returns amplitude 2.
ViStatus rsrth_QueryCursorAmplitude(ViSession instrSession,
                                    ViReal64* amplitude1,
                                    ViReal64* amplitude2)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_CURSOR_MEASUREMENT_AMPLITUDE_1, amplitude1),
            2, "Amplitude 1");

    viCheckParm(rsrth_GetAttributeViReal64(instrSession, NULL, RSRTH_ATTR_CURSOR_MEASUREMENT_AMPLITUDE_2, amplitude2),
            3, "Amplitude 2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument.  The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIFN     -
/// HIFN     *RST
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  OPC = -1;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsrth_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST"));
        checkErr(rsrth_QueryOPC(instrSession, &OPC));
    }

    checkErr(rsrth_DefaultInstrSetup(instrSession));
    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the revision numbers of the instrument driver
/// HIFN  and instrument firmware.
/// HIFN     RSSCOPE_ATTR_SPECIFIC_DRIVER_REVISION
/// HIFN     RSSCOPE_ATTR_INSTRUMENT_FIRMWARE_REVISION
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentDriverRevision/Returns the instrument driver software revision numbers in the form of
/// HIPAR instrumentDriverRevision/a string.
/// HIPAR firmwareRevision/Returns the instrument firmware revision numbers in the form of a
/// HIPAR firmwareRevision/string.
ViStatus rsrth_revision_query(ViSession instrSession,
                              ViChar    instrumentDriverRevision[],
                              ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads an error code and a message from the instrument's
/// HIFN  error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error
/// HIPAR errorMessage/message queue.
ViStatus rsrth_error_query(ViSession instrSession,
                           ViInt32*  errorCode,
                           ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus rsrth_error_message(ViSession instrSession,
                             ViStatus  errorCode,
                             ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;
    static   RsCoreStringValueTable errorTable =
             {
             RSRTH_ERROR_CODES_AND_MSGS,
    {0,      NULL}
             };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the endianess.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR byteOrder/Sets the endianess.
ViStatus rsrth_ByteOrder(ViSession instrSession,
                         ViInt32   byteOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_BYTE_ORDER, byteOrder),
            2, "Byte Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches ON/OFF various error checkings performed inside
/// HIFN  of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking (reading of
/// HIPAR statusChecking/the status byte and checking it for error). All driver functions are
/// HIPAR statusChecking/using state checking. Switch this function to ON when debugging your
/// HIPAR statusChecking/application. For better bus throughput and instrument performance
/// HIPAR statusChecking/switch it OFF.
ViStatus rsrth_ConfigureErrorChecking(ViSession instrSession,
                                      ViBoolean optionChecking,
                                      ViBoolean rangeChecking,
                                      ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, NULL, RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, NULL, RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus rsrth_SetOPCTimeout(ViSession instrSession,
                             ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus rsrth_GetOPCTimeout(ViSession instrSession,
                             ViUInt32* timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32*)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control sets the timeout value.
ViStatus rsrth_SetVISATimeout(ViSession instrSession,
                              ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control gets the timeout value.
ViStatus rsrth_GetVISATimeout(ViSession instrSession,
                              ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIFN     -
/// HIFN     *CLS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_WriteInstrData(instrSession, "*CLS"));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIFN     -
/// HIFN     *IDN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus rsrth_IDQueryResponse(ViSession instrSession,
                               ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIFN     -
/// HIFN     *WAI
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIFN     -
/// HIFN     *OPC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus rsrth_QueryOPC(ViSession instrSession,
                        ViInt32*  opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrth_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the date on the instrument.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR day/Day of date.
/// HIPAR month/Month of date.
/// HIPAR year/Year of date.
ViStatus rsrth_SystemDate(ViSession instrSession,
                          ViInt32   day,
                          ViInt32   month,
                          ViInt32   year)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:DATE %ld,%ld,%ld", day, month, year);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time on the instrument.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hours/Hours.
/// HIPAR minutes/Minutes.
/// HIPAR seconds/Seconds.
ViStatus rsrth_SystemTime(ViSession instrSession,
                          ViInt32   hours,
                          ViInt32   minutes,
                          ViInt32   seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:TIME %ld,%ld,%ld", hours, minutes, seconds);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the wireless LAN function of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the wireless LAN function of the instrument.
ViStatus rsrth_SystemWLANMode(ViSession instrSession,
                              ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WLAN_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIFN  automatically in CheckStatus() when Instrument Status Error is
/// HIFN  detected.
/// HIFN     Use the function rsdsa_GetError afterwards to get the detailed
/// HIFN  information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIPAR autoSystErrQuery/automatically in CheckStatus() when Instrument Status Error is
/// HIPAR autoSystErrQuery/detected.
ViStatus rsrth_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                           ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN  Session-related mutex switch.
/// HIFN     If TRUE (default after init is FALSE), the driver sets a
/// HIFN  mutual-exclusion mechanism for every driver's hi-level function and
/// HIFN  attribute access.
/// HIFN     Set this value to TRUE if you plan to use one session in more than one
/// HIFN  thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch.
ViStatus rsrth_ConfigureMultiThreadLocking(ViSession instrSession,
                                           ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN  This function configures attribute data type mismatch reporting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsdsa_init or
/// HIPAR instrSession/rsdsa_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables data type mismatch reporting.
ViStatus rsrth_ConfigureAttributeDataTypeMismatchReporting(ViSession instrSession,
                                                           ViBoolean state)
{  
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->reportAttrDataTypeMismatch = state;

Error:
    return error;
}

/// HIFN  This function retrieves and then clears the error information for the
/// HIFN  session or the current execution thread. One exception exists: If the
/// HIFN  BufferSize parameter is 0, the function does not clear the error
/// HIFN  information. By passing 0 for the buffer size, the caller can ascertain
/// HIFN  the buffer size required to get the entire error description string and
/// HIFN  then call the function again with a sufficiently large buffer.
/// HIFN     If the user specifies a valid session for the InstrumentHandle
/// HIFN  parameter, Get Error retrieves and then clears the error information
/// HIFN  for the session. If the user passes NULL for the InstrumentHandle
/// HIFN  parameter, this function retrieves and then clears the error
/// HIFN  information for the current execution thread.  If the InstrumentHandle
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error. Normally, the error information describes the first error
/// HIFN  that occurred since the user last called rsrth_GetError or
/// HIFN  rsrth_ClearError.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR code/Returns the error code for the session or execution thread.
/// HIPAR buffersize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR buffersize/Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If
/// HIPAR description/there is no description, the function returns an empty string.
ViStatus rsrth_GetError(ViSession instrSession,
                        ViStatus* errorCode,
                        ViInt32   bufferSize,
                        ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsrth_error_message, errorCode, bufferSize, description));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears the error code and error description for the
/// HIFN  session. If the user specifies a valid session for the InstrumentHandle
/// HIFN  parameter, this function clears the error information for the session.
/// HIFN  If the user passes NULL for the Vi parameter, this function clears
/// HIFN  the error information for the current execution thread. If the Vi
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error.
/// HIFN     The function clears the error code by setting it to VI_SUCCESS.  If
/// HIFN  the error description string is non-NULL, the function de-allocates the
/// HIFN  error description string and sets the address to NULL.
/// HIFN     Maintaining the error information separately for each thread is useful
/// HIFN  if the user does not have a session handle to pass to the
/// HIFN  rsrth_GetError function, which occurs when a call to rsrth_init or
/// HIFN  rsrth_InitWithOptions fails.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus rsrth_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN  This function writes a user-specified string to the instrument.
/// HIFN     Note:  This function bypasses IVI attribute state caching.  Therefore,
/// HIFN  when you call this function, the cached values for all attributes will
/// HIFN  be invalidated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus rsrth_WriteInstrData(ViSession instrSession,
                              ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus rsrth_QueryViBoolean(ViSession  instrSession,
                              ViString   command,
                              ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus rsrth_QueryViInt32(ViSession instrSession,
                            ViString  command,
                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus rsrth_QueryViReal64(ViSession instrSession,
                             ViString  command,
                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR value/Returns the string value.
ViStatus rsrth_QueryViString(ViSession instrSession,
                             ViString  command,
                             ViInt32   bufferSize,
                             ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1), 2, "Command");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, command, bufferSize, value, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sends a command to the instrument synchronised with OPC-polling. Use
/// HIFN  this function to wait for an instrument operation that can take a long
/// HIFN  time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value in ms. Set it to 0 if you don't
/// HIPAR timeout/want the function to wait until the acquisition is complete.
ViStatus rsrth_WriteCommandWithOPCSync(ViSession instrSession,
                                       ViString  writeBuffer,
                                       ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1), 2, "Write Buffer");

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sends a query to the instrument synchronised with OPC-polling. Use
/// HIFN  this function to wait for a response that can take a long time to
/// HIFN  complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value in ms. Set it to 0 if you don't
/// HIPAR timeout/want the function to wait until the acquisition is complete.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data that
/// HIPAR readBuffer/was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument and
/// HIPAR numBytesRead/stored in the Read Buffer.
ViStatus rsrth_QueryWithOPCSync(ViSession instrSession,
                                ViString  writeBuffer,
                                ViUInt32  timeout,
                                ViInt32   bufferSize,
                                ViChar    readBuffer[],
                                ViUInt32* numBytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  bytesRead = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1), 2, "Write Buffer");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, readBuffer), 5, "Read Buffer");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, &bytesRead));
    *numBytesRead = (ViUInt32)bytesRead;

    checkErr(RsCore_CheckStatus(instrSession, error));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function provides mass storage capabilities for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR source/This control specifies the source of the operation to be performed. If
/// HIPAR source/the operation requires a single parameter, this control specifies that
/// HIPAR source/parameter.
/// HIPAR destination/This control specifies the destination of the operation to be
/// HIPAR destination/performed.
ViStatus rsrth_FileManagerOperations(ViSession instrSession,
                                     ViInt32   operation,
                                     ViString  source,
                                     ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSRTH_VAL_FILE_COPY:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:COPY '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(RsCore_CheckStatus(instrSession, error));
        break;
    case RSRTH_VAL_FILE_MOVE:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:MOVE '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(RsCore_CheckStatus(instrSession, error));
        break;
    case RSRTH_VAL_FILE_DELETE:
        viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_DELETE_FILE, source),
                3, "Source");
        break;
    case RSRTH_VAL_FILE_NEW_DIR:
        viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_CREATE_DIRECTORY, source),
                3, "Source");
        break;
    case RSRTH_VAL_FILE_CDIR:
        viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_CURRENT_DIRECTORY, source),
                3, "Source");
        break;
    case RSRTH_VAL_FILE_RDIR:
        viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_REMOVE_DIRECTORY, source),
                3, "Source");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the current instrument settings to the specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR file/Stores the current instrument settings to the specified file.
ViStatus rsrth_FileSaveInstrumentSettings(ViSession instrSession,
                                          ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_SAVE_INSTRUMENT_SETTINGS, file),
            2, "File");

	checkErr(RsCore_CheckStatus(instrSession, error));
	
Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of files and subdirectories of the current or of a
/// HIFN  specified directory. The number includes the parent directory strings
/// HIFN  "." and ".." so that it corresponds to the number of strings returned
/// HIFN  by the rsrth_FileDirectoryContent function after the initial numeric
/// HIFN  parameters.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberofFilesAndFolders/Returns the number of files and subdirectories of the current or of a
/// HIPAR numberofFilesAndFolders/specified directory.
ViStatus rsrth_FileCount(ViSession instrSession,
                         ViString  directory,
                         ViInt32*  numberofFilesAndFolders)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if ((directory != NULL) && (strlen(directory) > 0))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG? '%s'", directory);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG?");
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberofFilesAndFolders));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the indicated directory. According to DOS
/// HIFN  convention, wild card characters can be entered in order to query eg. a
/// HIFN  list of all file of a certain type.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as comma
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus rsrth_FileDirectoryContent(ViSession instrSession,
                                    ViString  directory,
                                    ViInt32   bufferSize,
                                    ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, output),
        4, "Output");

    if ((directory != NULL) && (strlen(directory) > 0))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT? '%s'", directory);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT?");
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of subdirectories of the current or of a specified
/// HIFN  directory. The number includes the parent directory strings "." and
/// HIFN  ".." so that it corresponds to the number of strings returned by the
/// HIFN  rsrth_FileSubdirectories.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberofSubdirectories/Returns the number of files and subdirectories of the current or of a
/// HIPAR numberofSubdirectories/specified directory.
ViStatus rsrth_FileSubdirectoriesCount(ViSession instrSession,
                                       ViString  directory,
                                       ViInt32*  numberofSubdirectories)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if ((directory != NULL) && (strlen(directory) > 0))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DCAT:LENG? '%s'", directory);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DCAT:LENG?");
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberofSubdirectories));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the subdirectories of the current or of a specified directory.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as comma
/// HIPAR output/separated strings, for example: 'SPOOL1.WMF','SPOOL2.WMF','SPOOL3.WMF'
ViStatus rsrth_FileSubdirectories(ViSession instrSession,
                                  ViString  directory,
                                  ViInt32   bufferSize,
                                  ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, output),
        4, "Output");

    if ((directory != NULL) && (strlen(directory) > 0))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DCAT? '%s'", directory);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DCAT?");
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/instrument to control computer applies.
ViStatus rsrth_ReadToFileFromInstrument(ViSession instrSession,
                                        ViString  source,
                                        ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, source, 1, -1), 2, "Source");
    viCheckParm(RsCore_InvalidStringLength(instrSession, destination, 1, -1), 3, "Destination");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the host computer and write it
/// HIFN  to a user specified file in the instrument.
/// HIFN     -
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/control computer to instrument applies.
ViStatus rsrth_WriteFromFileToInstrument(ViSession instrSession,
                                         ViString  source,
                                         ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, source, 1, -1), 2, "Source");
    viCheckParm(RsCore_InvalidStringLength(instrSession, destination, 1, -1), 3, "Destination");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);

    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function restores the instrument settings from the specified
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR settingsFile/This control selects file for the instrument settings restore.
ViStatus rsrth_RecallInstrumentSettings(ViSession instrSession,
                                        ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_RECALL_INSTRUMENT_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the multichannel export.
/// HIFN     If enabled, select the channels to be exported using the
/// HIFN  rsrth_ConfigureWaveformExportChannel function.
/// HIFN     If disabled, select the waveform to be exported using the
/// HIFN  rsrth_ConfigureWaveformExportSource function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multichannelExport/Enables or disables the multichannel export.
ViStatus rsrth_ConfigureWaveformExportMultichannel(ViSession instrSession,
                                                   ViBoolean multichannelExport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT_MULTICHANNEL, multichannelExport),
            2, "Multichannel Export");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source waveform for export from the
/// HIFN  active waveforms of input channels, math signals and reference
/// HIFN  waveforms.
/// HIFN     RSSCOPE_ATTR_WAVEFORM_EXPORT_SOURCE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the waveform to be exported.
ViStatus rsrth_ConfigureWaveformExportSource(ViSession instrSession,
                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViInt32(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Encludes the history data in the export files. If acquisition is
/// HIFN  running, the command stops the acquisition.
/// HIFN     Note(s):
/// HIFN     (1) History must be enabled before using the command (CHAN:HIST:STAT
/// HIFN  ON).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataLogging/Enables the export of several subsequent waveforms into one file.
ViStatus rsrth_ConfigureWaveformExportScopeDataLogging(ViSession instrSession,
                                                       ViBoolean dataLogging)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT_DATA_LOGGING, dataLogging),
            2, "Data Logging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes horizontal values in the export data (time or frequency
/// HIFN  values, depending on the waveform). X and Y-values are written
/// HIFN  alternately to the file. If disabled, only Y-values - mostly voltage
/// HIFN  values - are written.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interleavedXY/Includes horizontal values in the export data.
ViStatus rsrth_WaveformExportInterleavedXY(ViSession instrSession,
                                           ViBoolean interleavedXY)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT_INTERLEAVED_XY, interleavedXY),
            2, "Interleaved XY");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the waveform(s) to the specified file name and
/// HIFN  path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR file/Stes the file name and path to save the waveform to.
ViStatus rsrth_WaveformExportFile(ViSession instrSession,
                                  ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT_FILE, file),
            2, "File");

    checkErr(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_WAVEFORM_EXPORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the path and filename of the user app. Make sure to load the app
/// HIFN  to the instrument before (rsrth_FileManagerOperations - copy file
/// HIFN  operation).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR appPath/Sets the path and filename of the user app. Make sure to load the app
/// HIPAR appPath/to the instrument before (RSRTH_FUNC_COPY_FILE).
ViStatus rsrth_ConfigureAppPath(ViSession instrSession,
                                ViString  appPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViString(instrSession, NULL, RSRTH_ATTR_APP_PATH, appPath),
            2, "App Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set the app persistence ON if the app contains commands that would
/// HIFN  otherwise reset the app, for example, *RST, :OP, mode changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR appPersistence/Set the app persistence ON if the app contains commands that would
/// HIPAR appPersistence/otherwise reset the app, for example, *RST, :OP, mode changes.
ViStatus rsrth_ConfigureAppPersistence(ViSession instrSession,
                                       ViBoolean appPersistence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrth_SetAttributeViBoolean(instrSession, NULL, RSRTH_ATTR_APP_PERSISTENCE, appPersistence),
            2, "App Persistence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrth_init or
/// HIPAR instrSession/rsrth_InitWithOptions function.
ViStatus rsrth_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}